-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009, 2010, 2011 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org


--Elemental strike damage types
newDamageType{
	name = "water strike", type = "WATER_STRIKE_BALL",
	projector = function(src, x, y, type, dam)
		local realdam = DamageType:get(DamageType.COLD).projector(src, x, y, DamageType.COLD, dam.dam)
		local target = game.level.map(x, y, Map.ACTOR)
		if target then
			-- Freeze it, if we pass the chance and stun test
			local sx, sy = game.level.map:getTileToScreen(x, y)
			if target:canBe("stun") and rng.percent(dam.chance) then
				target:setEffect(target.EFF_FROZEN, dam.dur, {hp=dam.hp, apply_power=src:combatPhysicalpower(), min_dur=1})
				game.flyers:add(sx, sy, 30, (rng.range(0,2)-1) * 0.5, -3, "Frozen!", {0,255,155})
			else
				game.flyers:add(sx, sy, 30, (rng.range(0,2)-1) * 0.5, -3, "Resist!", {0,255,155})
				game.logSeen(target, "%s resists!", target.name:capitalize())
			end
		end
		return realdam
	end,
}

newDamageType{
	name = "air strike", type = "AIR_STRIKE_BALL",
	projector = function(src, x, y, type, dam)
		local realdam = DamageType:get(DamageType.LIGHTNING).projector(src, x, y, DamageType.LIGHTNING, dam.dam)
		local target = game.level.map(x, y, Map.ACTOR)
		if target then
			-- Silence it if can be silenced and roll chance
			if target:canBe("silence") and rng.percent(dam.chance) then
				target:setEffect(target.EFF_SILENCED, math.ceil(dam.dur), {apply_power=src:combatPhysicalpower()})
			else
				game.logSeen(target, "%s resists!", target.name:capitalize())
			end
		end
		return realdam
	end,
}

newDamageType{
	name = "earth strike", type = "EARTH_STRIKE_BALL",
	projector = function(src, x, y, type, dam)
		local realdam = DamageType:get(DamageType.PHYSICAL).projector(src, x, y, DamageType.PHYSICAL, dam.dam)
		local target = game.level.map(x, y, Map.ACTOR)
		if target then
			-- Pin it if can be silenced and roll chance
			if target:canBe("pin") and rng.percent(dam.chance) then
				target:setEffect(target.EFF_PINNED, math.ceil(dam.dur), {apply_power=src:combatPhysicalpower()})
			else
				game.logSeen(target, "%s resists!", target.name:capitalize())
			end
		end
		return realdam
	end,
}

newDamageType{
	name = "fire strike", type = "FIRE_STRIKE_BALL",
	projector = function(src, x, y, type, dam)
		local realdam = DamageType:get(DamageType.FIRE).projector(src, x, y, DamageType.FIRE, dam.dam)
		local target = game.level.map(x, y, Map.ACTOR)
		if target then
			-- Debuff it with Purifying fire (dispel 1 effect/turn) it if can be silenced and roll chance
			if rng.percent(dam.chance) then
				target:setEffect(target.EFF_PURIFY_FIRE, math.ceil(dam.dur), {apply_power=src:combatPhysicalpower()})
			else
				game.logSeen(target, "%s resists!", target.name:capitalize())
			end
		end
		return realdam
	end,
}

newDamageType{
	name = "mark of fire", type = "MARK_OF_FIRE",
	projector = function(src, x, y, type, dam)
		local target = game.level.map(x, y, Map.ACTOR)
		if target then
			if src:reactionToward(target) < 0 then
				target:setEffect(target.EFF_MARK_OF_FIRE, 1, {power=dam})
			end
		end
	end,
}

