-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009, 2010, 2011 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

newTalent{
	name = "Fire Strike",
	type = {"spell/blazebrand", 1},
	require = spells_req1,
	points = 5,
	random_ego = "attack",
	cooldown = 8,
	stamina = 12,
	tactical = { ATTACK = { weapon = 2 }, DISABLE = { pin = 2 } },
	getDuration = function(self,t) return math.floor(1+math.floor(self:getTalentLevel(t)/2)) end,
	radius = function(self, t)
		return math.floor(self:getTalentLevelRaw(t)/2)
	end,
	action = function(self, t)
		--figure out which weapon (or none) to use
		local weapon = nil
		if self:getInven("MAINHAND") then
			weapon = self:getInven("MAINHAND")[1]
			if not weapon or not weapon.combat then
				weapon = nil
			end
		end
		if self:attr("disarmed") or not weapon then
			weapon = self.combat
		end

		local tg = {type="hit", range=self:getTalentRange(t)}
		local x, y, target = self:getTarget(tg)
		if not x or not y then return nil end
		if target then
			if core.fov.distance(self.x, self.y, x, y) > 1 then return nil end
			local speed, hit = self:attackTargetWith(target, weapon.combat, nil, self:combatTalentWeaponDamage(t, 1, 1.5))
			-- Try to purify !
			--if hit then
			--	target:setEffect(target.EFF_PURIFY_FIRE, t.getDuration(self,t), {apply_power=self:combatPhysicalpower(), min_dur=1})
			--end
		end
		--Even if we miss or no target, the power must explode in a ball!
		local ball_radius = t.radius(self,t)
		local tg = {type="ball", radius=ball_radius, selffire=false, friendlyfire=false, talent=t}
		self:project(tg, x, y, DamageType.FIRE_STRIKE_BALL,{dur=t.getDuration(self,t), chance=100, dam=self:combatTalentPhysicalDamage(t,10,150)})
		if ball_radius > 0 then
			game.level.map:particleEmitter(x, y, tg.radius, "ball_fire", {radius=tg.radius, tx=x, ty=y})
		end

		if not self:isTalentActive(self.T_FIRE_STANCE) then self:forceUseTalent(self.T_FIRE_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true}) end
		return true
	end,
	--Learn and unlearn effects, add/remove fire stance
	on_learn = function(self, t)
		local lev = self:getTalentLevelRaw(t)
		if lev == 1 then
			self:learnTalent(self.T_FIRE_STANCE, true, nil, {no_unlearn=true})
		end
	end,
	on_unlearn = function(self, t)
		local lev = self:getTalentLevelRaw(t)
		if lev == 0 then
			self:unlearnTalent(self.T_FIRE_STANCE)
		end
	end,

	info = function(self, t)
		return ([[Strike with the power of Fire. The user strikes with their mainhand weapon, dealing %d%% damage and applying the purifying fire effect.
		Purifying fire causes a beneficial effect or sustain to be removed each turn for %d turns.
		It also creates an explosive ball of fire of radius %d, centered on the target, and which deals %0.2f fire damage and applies the purifying fire effect to enemies.
		User's physical power opposes target's spell save to determine if purifying fire is applied.
		After use, the user activates Fire Stance.  This talent may be used without a target.]]):
		format(100 * self:combatTalentWeaponDamage(t, 1, 1.5), t.getDuration(self,t), t.radius(self,t), self:combatTalentPhysicalDamage(t,10,150))
	end,
}

newTalent{
	name = "Inferno Cleave",
	type = {"spell/blazebrand", 2},
	require = spells_req2,
	points = 5,
	random_ego = "attack",
	cooldown = 12,
	stamina = 25,
	tactical = { ATTACKAREA = { weapon = 1, fire = 1 } },
	getFireDamage = function(self, t) return self:combatTalentPhysicalDamage(t,10,100) end,
	action = function(self, t)
		local combatType = nil
		if not self:attr("disarmed") and not self:isUnarmed() then
			-- All weapons in main hands
			if self:getInven(self.INVEN_MAINHAND) then
				for i, o in ipairs(self:getInven(self.INVEN_MAINHAND)) do
					if o.combat and not o.archery then
						combatType = o.combat
					end
				end
			end
		end
		-- Barehanded ?
		if not combatType and self.combat then
			combatType = self.combat
		end

		local statmod = 0.6
		local dammod = combatType.dammod or {str=0.6}
		for stat, mod in pairs(combatType.dammod) do
			if stat == "str" then
				statmod = mod
			end
		end

		local tg = {type="hit", range=self:getTalentRange(t)}
		local x, y, target = self:getTarget(tg)
		if not x or not y or not target then return nil end
		if core.fov.distance(self.x, self.y, x, y) > 1 then return nil end

		local dir = util.getDir(x, y, self.x, self.y)
		if dir == 5 then return nil end
		local lx, ly = util.coordAddDir(self.x, self.y, util.dirSides(dir, self.x, self.y).left)
		local rx, ry = util.coordAddDir(self.x, self.y, util.dirSides(dir, self.x, self.y).right)
		local lt, rt = game.level.map(lx, ly, Map.ACTOR), game.level.map(rx, ry, Map.ACTOR)

		local hit
		local s
		s, hit = self:attackTargetWith(target, combatType, nil,self:combatTalentWeaponDamage(t, 1, 1.5) )
		self:project(tg, x, y, DamageType.LITE, 1)
		if hit then self:project(tg, x, y, DamageType.FIREBURN, t.getFireDamage(self,t) * statmod) end
		if self:getTalentLevel(t) >= 3 and target:canBe("blind") then self:project(tg, x, y, DamageType.BLIND, 2) end

		if lt then
			s, hit = self:attackTargetWith(lt, combatType, nil,self:combatTalentWeaponDamage(t, 1, 1.5) )
			self:project(tg, lx, ly, DamageType.LITE, 1)
			if hit then self:project(tg, lx, ly, DamageType.FIREBURN, t.getFireDamage(self,t) * statmod) end
			if self:getTalentLevel(t) >= 3 and target:canBe("blind") then self:project(tg, lx, ly, DamageType.BLIND, 2) end
		end

		if rt then
			s, hit = self:attackTargetWith(rt, combatType, nil,self:combatTalentWeaponDamage(t, 1, 1.5) )
			self:project(tg, lx, ly, DamageType.LITE, 1)
			if hit then self:project(tg, lx, ly, DamageType.FIREBURN, t.getFireDamage(self,t) * statmod) end
			if self:getTalentLevel(t) >= 3 and target:canBe("blind") then self:project(tg, lx, ly, DamageType.BLIND, 2) end

		end
		print(x,y,target)
		print(lx,ly,lt)
		print(rx,ry,rt)

		return true
	end,
	--Increase/Decrease spell/fire mastery by 0.03
	--on_learn = function(self, t)
	--	self:setTalentTypeMastery("spell/fire", self:getTalentTypeMastery("spell/fire") + 0.03)
	--end,
	--on_unlearn = function(self, t)
	--	self:setTalentTypeMastery("spell/fire", self:getTalentTypeMastery("spell/fire") - 0.03)
	--end,
	info = function(self, t)
		local dam = t.getFireDamage(self,t)
		return ([[This talent cleaves through up to 2 additional targets next to the primary target with the user's mainhand weapon, dealing %d%% damage.
		In addition targets hit will be set on fire, dealing %0.2f damage over 3 turns, modified by weapon type and physical power.
		Squares the targets are in will be lit up.  At level 3 it will also blind enemies for 2 turns.]]):
		format(100 * self:combatTalentWeaponDamage(t, 1, 1.5),dam)
	end,
}

newTalent{
	name = "Aura of Fire",
	type = {"spell/blazebrand", 3},
	require = spells_req3,
	points = 5,
	mode = "sustained",
	cooldown = 16,
	sustain_mana = 30,
	no_energy = true,
	tactical = { BUFF = 2, CURE = 2 },
	getLifeCost = function(self,t) return (8 - math.min(self:getTalentLevelRaw(t),5)) / 100 end,
	getDamage = function(self,t) return self:combatTalentSpellDamage(t,1,50) end,
	getTarget = function(self, t)
		return {type="ball", range=0, radius=1, selffire=false, friendlyfire=false}
	end,
	do_aura_of_fire = function(self,t)
		if (self.life / self.max_life) < 0.2 then
			self:forceUseTalent(self.T_AURA_OF_FIRE, {ignore_energy=true, ignore_ressources=true})
		else

			local effs = {}
			-- Go through all spell effects
			for eff_id, p in pairs(self.tmp) do
				local e = self.tempeffect_def[eff_id]
				if e.status == "detrimental" and e.type ~= "other" then
					effs[#effs+1] = {"effect", eff_id}
				end
			end

			if #effs > 0 then
				local eff = rng.tableRemove(effs)
				if eff[1] == "effect" then
					self:removeEffect(eff[2])
					self:takeHit(self.max_life*t.getLifeCost(self,t),self)
					game.logSeen(self, "%s is purified as %d life burns away!",self.name:capitalize(),self.max_life*t.getLifeCost(self,t))
				end
			end
			local tg = t.getTarget(self, t)
			self:project(tg,self.x,self.y,DamageType.FIRE,t.getDamage(self,t))
		end
	end,
	activate = function(self,t)
		return {
			particle = self:addParticles(Particles.new("wildfire", 1)),
		}
	end,
	deactivate = function(self,t,p)
		self:removeParticles(p.particle)
		return true
	end,
	--Increase/Decrease spell/fire mastery by 0.03
	--on_learn = function(self, t)
	--	self:setTalentTypeMastery("spell/fire", self:getTalentTypeMastery("spell/fire") + 0.03)
	--end,
	--on_unlearn = function(self, t)
	--	self:setTalentTypeMastery("spell/fire", self:getTalentTypeMastery("spell/fire") - 0.03)
	--end,
	info = function(self, t)
		local dam = t.getDamage(self,t)
		local lifeCost = t.getLifeCost(self,t)
		return ([[Surrounds the caster in an aura of fire.  It will deal %0.2f fire damage to all adjacent enemies each turn.
		When affected by detrimental effects, the aura will remove one effect each turn at a cost of %d%% of the user's max life.
		Aura of fire will shut off if the user's life falls below 20%% while affected by detrimental effects.]]):
		format(dam,lifeCost * 100)
	end,
}


newTalent{
	name = "Mark of Fire",
	type = {"spell/blazebrand", 4},
	require = spells_req4,
	points = 5,
	random_ego = "attack",
	cooldown = 20,
	mana = 25,
	no_energy = true,
	range = 0,
	radius = function(self,t) return math.floor(self:getTalentLevel(t)) end,
	target = function(self, t)
		return {type="ball", range=self:getTalentRange(t), radius=self:getTalentRadius(t)}
	end,
	getDuration = function(self, t) return math.floor(self:getTalentLevel(t)) end,
	action = function(self,t)
		local tg = self:getTalentTarget(t)
		self:project(tg, self.x, self.y, DamageType.LITE, 1)
		self:project(tg, self.x, self.y, DamageType.MARK_OF_FIRE, 1.5)
		-- Add a lasting map effect
		game.level.map:addEffect(self,
			self.x, self.y, t.getDuration(self, t),
			DamageType.MARK_OF_FIRE, 1.5,
			self:getTalentRadius(t),
			5, nil,
			{type="inferno"},
			nil, true
		)
		game:playSoundNear(self, "talents/fire")
		return true
	end,
	--Increase/Decrease spell/fire mastery by 0.03
	--on_learn = function(self, t)
	--	self:setTalentTypeMastery("spell/fire", self:getTalentTypeMastery("spell/fire") + 0.03)
	--end,
	--on_unlearn = function(self, t)
	--	self:setTalentTypeMastery("spell/fire", self:getTalentTypeMastery("spell/fire") - 0.03)
	--end,
	info = function(self, t)
		return ([[Create a mystical mark of fire on the ground of radius %d lasting for %d turns.
		All enemies standing upon the mark will take an additional 50%% more damage from all sources and have their movement slowed by 50%%.]]):
		format(math.floor(self:getTalentLevel(t)),math.floor(self:getTalentLevel(t)))
	end,
}

newTalent{
	name = "Fire Stance",
	type = {"spell/elemental-stance", 1},
	points = 1,
	mode = "sustained",
	cooldown = 0,
	elemental_stance = true,
	getStancePhysMod = function(self,t) return math.ceil(self:combatTalentSpellDamage(t,10,50)) end,
	getStanceCritMod = function(self,t) return self:combatTalentSpellDamage(t,2,20) end,
	on_pre_use = function(self, t, silent)
		-- prevent anyone from activating other than without the "force_talent_ignore_ressources" flag
		if not self:attr("force_talent_ignore_energy") then
			if not silent then game.logPlayer(self, "You can not active Fire Stance directly.  Use Fire Strike to activate it.") end
			return false
		end
		return true
	end,
	activate = function(self,t)
		local stance_turned_off = nil
		self:attr("fire_stance", 1)
		if self:knowTalent(self.T_WATER_STANCE) and self:isTalentActive(self.T_WATER_STANCE) then
			self:forceUseTalent(self.T_WATER_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if self:knowTalent(self.T_AIR_STANCE) and self:isTalentActive(self.T_AIR_STANCE) then
			self:forceUseTalent(self.T_AIR_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if self:knowTalent(self.T_EARTH_STANCE) and self:isTalentActive(self.T_EARTH_STANCE) then
			self:forceUseTalent(self.T_EARTH_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if not stance_turned_off and self:knowTalent(self.T_ELEMENTAL_DESTRUCTION) and self:isTalentActive(self.T_ELEMENTAL_DESTRUCTION) then
			self:forceUseTalent(self.T_ELEMENTAL_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			self:forceUseTalent(self.T_ELEMENTAL_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
		end
		return {
			dam = self:addTemporaryValue("combat_dam", t.getStancePhysMod(self,t)),
			crit = self:addTemporaryValue("combat_physcrit", t.getStanceCritMod(self,t))
		}
	end,
	deactivate = function(self,t,p)
		self:attr("fire_stance",-1)
		--turn Arcane Destruction on and off if on.
		if self:isTalentActive(self.T_ELEMENTAL_DESTRUCTION) then
			self:forceUseTalent(self.T_ELEMENTAL_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			self:forceUseTalent(self.T_ELEMENTAL_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
		end
		self:removeTemporaryValue("combat_dam", p.dam)
		self:removeTemporaryValue("combat_physcrit", p.crit)
		return true
	end,
	info = function(self, t)
		local dam = t.getStancePhysMod(self,t)
		local crit = t.getStanceCritMod(self,t)
		return ([[The user may only enter the elemental stance of Fire by using the Fire Strike talent.
		Only one stance can be active at a time.
		Fire Stance increases the user's physical power by %d and physical critical chance by %0.2f%%.
		Benefits increase with the Elemental Stance talent and Spell power.]])
		:format(dam,crit)
	end,
}
