-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009, 2010, 2011 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

newTalent{
	name = "Water Strike",
	type = {"spell/lifetide", 1},
	require = spells_req1,
	points = 5,
	random_ego = "attack",
	cooldown = 8,
	stamina = 12,
	tactical = { ATTACK = { weapon = 2 }, DISABLE = { stun = 2 } },
	getDuration = function(self,t) return math.floor(1+math.floor(self:getTalentLevel(t)/2)) end,
	radius = function(self, t)
		return math.floor(self:getTalentLevelRaw(t)/2)
	end,
	action = function(self, t)
		--figure out which weapon (or none) to use
		local weapon = nil
		local offweapon = nil
		if self:getInven("MAINHAND") then
			weapon = self:getInven("MAINHAND")[1]
			if not weapon or not weapon.combat then
				weapon = nil
			end
		end
		if self:getInven("OFFHAND") then
			offweapon = self:getInven("OFFHAND")[1]
			if not offweapon or not offweapon.combat then
				offweapon = nil
			end
			if not weapon then
				weapon = offweapon
				offweapon = nil
			end
		end
		if self:attr("disarmed") or not weapon then
			weapon = self.combat
			offweapon = nil
		end

		local tg = {type="hit", range=self:getTalentRange(t)}
		local x, y, target = self:getTarget(tg)
		if not x or not y then return nil end
		if target then
			if core.fov.distance(self.x, self.y, x, y) > 1 then return nil end

			--If dual wielding, attack with second weapon first, so as to not affect the ice block
			if offweapon then
				local speed, hit = self:attackTargetWith(target, offweapon.combat, nil, self:combatTalentWeaponDamage(t, 1, 1.5))
			end

			local speed, hit = self:attackTargetWith(target, weapon.combat, nil, self:combatTalentWeaponDamage(t, 1, 1.5))
			-- Try to stun !
			--if hit then
			--	if target:canBe("stun") then
			--		target:setEffect(target.EFF_FROZEN, t.getDuration(self,t), {hp=70 + self:combatPhysicalpower() * self:getTalentLevel(t), apply_power=self:combatPhysicalpower(), min_dur=1})
			--		game.flyers:add(x, y, 30, (rng.range(0,2)-1) * 0.5, -3, "Frozen!", {0,255,155})
			--	else
			--		game.flyers:add(x, y, 30, (rng.range(0,2)-1) * 0.5, -3, "Resist!", {0,255,155})
			--		game.logSeen(target, "%s resists!", target.name:capitalize())
			--	end
			--end
		end
		--The power must explode in a ball!
		local ball_radius = t.radius(self,t)
		local hp = 70 + self:combatPhysicalpower() * self:getTalentLevel(t)
		local tg = {type="ball", radius=t.radius(self,t), selffire=false, friendlyfire=false, talent=t}
		self:project(tg, x, y, DamageType.WATER_STRIKE_BALL,{dur=t.getDuration(self,t), chance=100, hp=hp, dam=self:combatTalentPhysicalDamage(t,10,150)})
		if ball_radius > 0 then
			game.level.map:particleEmitter(x, y, tg.radius, "ball_ice", {radius=tg.radius, tx=x, ty=y})
		end

		if not self:isTalentActive(self.T_WATER_STANCE) then self:forceUseTalent(self.T_WATER_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true}) end
		return true
	end,
	--Learn and unlearn effects, add/remove water stance
	on_learn = function(self, t)
		local lev = self:getTalentLevelRaw(t)
		if lev == 1 then
			self:learnTalent(self.T_WATER_STANCE, true, nil, {no_unlearn=true})
		end
	end,
	on_unlearn = function(self, t)
		local lev = self:getTalentLevelRaw(t)
		if lev == 0 then
			self:unlearnTalent(self.T_WATER_STANCE)
		end
	end,

	info = function(self, t)
		return ([[Strike with the power of Water. The user strikes with all their weapons, each dealing %d%% damage and freezing the target for %d turns.
		It also creates an explosive ball of water of radius %d, centered on the target, which deals %0.2f cold damage and freezes enemies.
		User's physical power opposes target's physical save to determine freezing.
		After use, the user activates Water Stance.  This talent may be used without a target.]]):
		format(100 * self:combatTalentWeaponDamage(t, 1, 1.5), t.getDuration(self,t), t.radius(self,t), self:combatTalentPhysicalDamage(t,10,150))
	end,
}

newTalent{
	name = "Healing Waters",
	type = {"spell/lifetide", 2},
	require = spells_req2,
	points = 5,
	mana = 25,
	cooldown = 16,
	tactical = { HEAL = 2 },
	getHeal = function(self, t) return 20 + self:combatTalentSpellDamage(t, 20, 500) end,
	getDuration = function(self,t) return math.ceil(self:getTalentLevel(t)/2) end,
	is_heal = true,
	is_spell = true,
	action = function(self, t)
		local healTotal = t.getHeal(self,t)
		local healingNeeded = (self.max_life - self.life) / util.bound((self.healing_factor or 1),0.1,2.5)
		self:heal(math.min(healTotal,healingNeeded), self)
		if healTotal - healingNeeded > 0 then
			self:setEffect(self.EFF_HEALING_WATERS, t.getDuration(self,t), {power= (healTotal - healingNeeded)})
		end
		game:playSoundNear(self, "talents/heal")
		return true
	end,
	info = function(self, t)
		local heal = t.getHeal(self, t)
		local dur = t.getDuration(self,t)
		return ([[Healing waters flow over you, healing your body for %d life.  Excess healing power will remain for %d turns or until used.
		The life healed will increase with your spell power.]]):
		format(heal,dur)
	end,
}

newTalent{
	name = "Tidal Rush",
	type = {"spell/lifetide", 3},
	require = spells_req3,
	points = 5,
	random_ego = "attack",
	cooldown = 18,
	stamina = 25,
	range = function(self,t) return math.min(3 + self:getTalentLevelRaw(t),10) end,
	tactical = { CLOSEIN = 2 },
	requires_target = true,
	waveTarget = function(self, t)
		return {type="ball", range=0, radius=1, selffire=false, talent=t}
	end,
	getWaveDamage = function(self, t) return self:combatTalentPhysicalDamage(t, 10, 150) end,
	getPinDuration = function(self,t) return math.ceil(self:getTalentLevel(t)/2) end,
	action = function(self, t)
		local targeting = {type="bolt", range=self:getTalentRange(t), nolock=true}
		local targetX, targetY, actualTarget = self:getTarget(targeting)

		if not targetX or not targetY then return nil end
		if core.fov.distance(self.x, self.y, targetX, targetY) > self:getTalentRange(t) then return nil end
		if core.fov.distance(self.x, self.y, targetX, targetY) < 2 then
			game.logSeen(self, "Tidal Rush's target is too close!  It must be at least 2 spaces away.")
			return nil
		end

		local block_actor = function(_, bx, by) return game.level.map:checkEntity(bx, by, Map.TERRAIN, "block_move", target) end
		local lineFunction = core.fov.line(self.x, self.y, targetX, targetY, block_actor)
		local nextX, nextY, is_corner_blocked = lineFunction:step()
		local moved = false
		--Ok, start to loop (emergency break conditions included if we somehow run out of next's or blocked)
		while nextX and nextY and not is_corner_blocked do

			--First we mini-wave against all targets in a ball of radius 1
			local tgwave = t.waveTarget(self,t)
			self:project(tgwave, self.x, self.y, DamageType.WAVE, {dam=self:spellCrit(t.getWaveDamage(self, t)),x=self.x,y=self.y,dist=1})
			game.level.map:particleEmitter(self.x, self.y, tgwave.radius, "ball_ice", {radius=tgwave.radius})
			local blockingTarget = game.level.map(nextX, nextY, Map.ACTOR)
			moved = false
			--If someone in the way, swap positions with them - the undertow pulls them away!
				if blockingTarget then
					-- Displace
					local tx,ty,sx,sy = blockingTarget.x,blockingTarget.y, self.x, self.y
					self.x = nil self.y = nil
					self:move(tx, ty, true)
					blockingTarget.x = nil blockingTarget.y = nil
					blockingTarget:move(sx, sy, true)
					game.logSeen(self, "%s is pulled away by the water's undertow!", blockingTarget.name)
					moved = true
				end
			if not moved then
				-- check that we can move
				if not game.level.map:isBound(nextX, nextY) or game.level.map:checkAllEntities(nextX, nextY, "block_move", self) then break end

				--do the move
				self:move(nextX,nextY, true)
			end
			--if now adjacent to target, break, or else hand over the next position
			if core.fov.distance(self.x, self.y, targetX, targetY) == 1 then
				break
			else
				nextX, nextY, is_corner_blocked = lineFunction:step()
			end
		end
		--We are now presumably next to the target, or something has gone horribly wrong
		--Attack the final target
		if actualTarget then
			if core.fov.distance(self.x, self.y, actualTarget.x, actualTarget.y) <= 1 then
				local tg = {type="hit", range=1}
				self:project(tg, actualTarget.x, actualTarget.y, DamageType.COLDNEVERMOVE, {dur=t.getPinDuration(self,t), dam=self:spellCrit(t.getWaveDamage(self, t))})
				self:attackTarget(actualTarget, nil, self:combatTalentWeaponDamage(t, 1, 1.5), true)
			end
		end
		game:playSoundNear(self, "talents/tidalwave")
		return true
	end,
	info = function(self, t)
		local dam = t.getWaveDamage(self,t)
		local dur = t.getPinDuration(self,t)
		return ([[The user summons a huge tide and rides it to the target, dealing %d%% damage with all weapons, plus %0.2f cold damage which freezes the target to ground, preventing movement for %d turns.
		Enemies nearby maybe knocked back or switch positions with the user on the way to the target.
		The sheer force of the wave deals %0.2f cold and physical damage.
		Damage, chance to knockback, and chance to pin increase with physical power.]]):format(100 * self:combatTalentWeaponDamage(t, 1, 1.5),dam,dur,dam)
	end,
}


newTalent{
	name = "Drain Water of Life",
	type = {"spell/lifetide", 4},
	require = spells_req4,
	points = 5,
	random_ego = "attack",
	cooldown = 16,
	mana = 35,
	tactical = { ATTACK = { weapon = 2 }, HEAL = 2 },
	action = function(self, t)
		local tg = {type="hit", range=self:getTalentRange(t)}
		local x, y, target = self:getTarget(tg)
		if not x or not y then return nil end
		if target then
			if core.fov.distance(self.x, self.y, x, y) > 1 then return nil end

			self:attr("drain_water_of_life", 1)
			self:attackTarget(target, nil, self:combatTalentWeaponDamage(t, 1.2, 2.1), true)
			self:attr("drain_water_of_life", -1)
		end
		return true
	end,
	info = function(self, t)
		return ([[The user strike with all weapons, dealing %d%% damage.  The user heals %d%% of the damage dealt to the target.]]):
		format(100 * self:combatTalentWeaponDamage(t, 1.2, 2.1),math.min(self:getTalentLevelRaw(self.T_DRAIN_WATER_OF_LIFE) * 20,100))
	end,
}


newTalent{
	name = "Water Stance",
	type = {"spell/elemental-stance", 1},
	points = 1,
	mode = "sustained",
	cooldown = 0,
	elemental_stance = true,
	getStanceHealMod = function(self,t) return self:combatTalentSpellDamage(t,5,40)/100 end,
	getStanceHealPerHit = function(self,t) return self:combatTalentSpellDamage(t,2,20) end,
	on_pre_use = function(self, t, silent)
		-- prevent anyone from activating other than without the "force_talent_ignore_ressources" flag
		if not self:attr("force_talent_ignore_energy") then
			if not silent then game.logPlayer(self, "You can not active Water Stance directly.  Use Water Strike to activate it.") end
			return false
		end
		return true
	end,
	do_trigger = function(self,t)
		self:heal(t.getStanceHealPerHit(self,t))
	end,
	activate = function(self,t)
		local stance_turned_off = nil
		self:attr("water_stance", 1)
		if self:knowTalent(self.T_AIR_STANCE) and self:isTalentActive(self.T_AIR_STANCE) then
			self:forceUseTalent(self.T_AIR_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if self:knowTalent(self.T_EARTH_STANCE) and self:isTalentActive(self.T_EARTH_STANCE) then
			self:forceUseTalent(self.T_EARTH_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if self:knowTalent(self.T_FIRE_STANCE) and self:isTalentActive(self.T_FIRE_STANCE) then
			self:forceUseTalent(self.T_FIRE_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if not stance_turned_off and self:knowTalent(self.T_ELEMENTAL_DESTRUCTION) and self:isTalentActive(self.T_ELEMENTAL_DESTRUCTION) then
			self:forceUseTalent(self.T_ELEMENTAL_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			self:forceUseTalent(self.T_ELEMENTAL_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
		end
		return {
			heal_inc = self:addTemporaryValue("healing_factor", t.getStanceHealMod(self,t)),
		}
	end,
	deactivate = function(self,t,p)
		self:attr("water_stance", -1)
		--turn Arcane Destruction on and off if on.
		if self:isTalentActive(self.T_ELEMENTAL_DESTRUCTION) then
			self:forceUseTalent(self.T_ELEMENTAL_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			self:forceUseTalent(self.T_ELEMENTAL_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
		end
		self:removeTemporaryValue("healing_factor", p.heal_inc)
		return true
	end,
	info = function(self, t)
		local healinc = t.getStanceHealMod(self, t)
		local regeninc = t.getStanceHealPerHit(self,t)
		return ([[The user may only enter the elemental stance of Water by using the Water Strike talent.
		Only one stance can be active at a time.
		Water Stance increases the user's healing effects by %d%% and heals %0.2f damage per melee hit.
		Benefits increase with the Elemental Stance talent and spell power.]])
		:format(healinc * 100, regeninc)
	end,
}
