-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009, 2010, 2011 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

newTalent{
	name = "Earth Strike",
	type = {"spell/stoneshield", 1},
	require = spells_req1,
	points = 5,
	random_ego = "attack",
	cooldown = 8,
	stamina = 12,
	tactical = { ATTACK = { weapon = 2 }, DISABLE = { pin = 2 } },
	getDuration = function(self,t) return math.floor(1+math.floor(self:getTalentLevel(t)/2)) end,
	radius = function(self, t)
		return math.floor(self:getTalentLevelRaw(t)/2)
	end,
	action = function(self, t)
		local shield = self:hasShield()
		--figure out which weapon (or none) to use
		local weapon = nil
		if self:getInven("MAINHAND") then
			weapon = self:getInven("MAINHAND")[1]
			if not weapon or not weapon.combat then
				weapon = nil
			end
		end
		if self:attr("disarmed") or not weapon then
			weapon = self.combat
		end

		local tg = {type="hit", range=self:getTalentRange(t)}
		local x, y, target = self:getTarget(tg)
		if not x or not y then return nil end
		if target then
			if core.fov.distance(self.x, self.y, x, y) > 1 then return nil end
			local speed, hit = self:attackTargetWith(target, weapon.combat, nil, self:combatTalentWeaponDamage(t, 1, 1.5))
			-- Try to pin !
			--if hit then
			--	if target:canBe("pin") then
			--		target:setEffect(target.EFF_PINNED, t.getDuration(self,t), {apply_power=self:combatPhysicalpower()})
			--	else
			--		game.logSeen(target, "%s resists!", target.name:capitalize())
			--	end
			--end
			--If wielding a shield, attack with it as a second weapon
			if shield then
				local speed, hit = self:attackTargetWith(target, shield.special_combat, nil, self:combatTalentWeaponDamage(t, 1, 1.5))
			end
		end
		--Even if we miss or no target, the power must explode in a ball!
		local ball_radius = t.radius(self,t)
		local tg = {type="ball", radius=ball_radius, selffire=false, friendlyfire=false,talent=t}
		self:project(tg, x, y, DamageType.EARTH_STRIKE_BALL,{dur=t.getDuration(self,t), chance=100, dam=self:combatTalentPhysicalDamage(t,10,150)})
		if ball_radius > 0 then
			game.level.map:particleEmitter(x, y, tg.radius, "ball_earth", {radius=tg.radius, tx=x, ty=y})
		end

		if not self:isTalentActive(self.T_EARTH_STANCE) then self:forceUseTalent(self.T_EARTH_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true}) end
		return true
	end,
	--Learn and unlearn effects, add/remove earth stance
	--Increase/Decrease spell/earth mastery by 0.03
	on_learn = function(self, t)
		local lev = self:getTalentLevelRaw(t)
		if lev == 1 then
			self:learnTalent(self.T_EARTH_STANCE, true, nil, {no_unlearn=true})
		end
	--	self:setTalentTypeMastery("spell/earth", self:getTalentTypeMastery("spell/earth") + 0.03)
	end,
	on_unlearn = function(self, t)
		local lev = self:getTalentLevelRaw(t)
		if lev == 0 then
			self:unlearnTalent(self.T_EARTH_STANCE)
		end
	--	self:setTalentTypeMastery("spell/earth", self:getTalentTypeMastery("spell/earth") - 0.03)
	end,
	info = function(self, t)
		return ([[Strike with the power of Earth. The user strikes with their mainhand weapon and bashes with their shield (if worn), dealing %d%% damage with each and pinning the target for %d turns.
		It also creates an explosive ball of stone of radius %d, centered on the target, which deals %0.2f physical damage and pins enemies.
		User's physical power opposes target's physical save to determine pinning.
		After use, the user activates Earth Stance.  This talent may be used without a target.]]):
		format(100 * self:combatTalentWeaponDamage(t, 1, 1.5),t.getDuration(self,t), t.radius(self,t),self:combatTalentPhysicalDamage(t,10,150))
	end,
}


newTalent{
	name = "Sandblast",
	type = {"spell/stoneshield", 2},
	require = spells_req2,
	points = 5,
	random_ego = "attack",
	cooldown = 10,
	stamina = 12,
	tactical = { ATTACK = { weapon = 2 }, DISABLE = { blind = 2 } },
	getDuration = function(self,t) return math.floor(self:getTalentLevelRaw(t)) end,
	getDamage = function(self,t) return self:combatTalentPhysicalDamage(t,10,100) end,
	action = function(self, t)
		local shield = self:hasShield()
		--figure out which weapon (or none) to use
		local weapon = nil
		if self:getInven("MAINHAND") then
			weapon = self:getInven("MAINHAND")[1]
			if not weapon or not weapon.combat then
				weapon = nil
			end
		end
		if self:attr("disarmed") or not weapon then
			weapon = self.combat
		end

		local tg = {type="hit", range=self:getTalentRange(t)}
		local x, y, target = self:getTarget(tg)
		if not x or not y then return nil end
		if target then
			if core.fov.distance(self.x, self.y, x, y) > 1 then return nil end
			local speed, hit = self:attackTargetWith(target, weapon.combat, nil, self:combatTalentWeaponDamage(t, 1, 1.5))
			--If wielding a shield, attack with it as a second weapon
			if shield then
				local speed, hit = self:attackTargetWith(target, shield.special_combat, nil, self:combatTalentWeaponDamage(t, 1, 1.5))
			end
		end
		--Now blind effect!
		local dur = t.getDuration(self,t)
		local tgsand = {type="cone", radius=3, selffire=false, friendlyfire=false}
		self:project(tgsand, x, y, DamageType.SAND, {dam=t.getDamage(self,t),dur=dur})
		game.level.map:particleEmitter(self.x, self.y, tgsand.radius, "breath_earth", {radius=tgsand.radius, tx=x-self.x, ty=y-self.y})
		game:playSoundNear(self, "talents/earth")
		return true
	end,
	--Increase/Decrease spell/earth mastery by 0.03
	--on_learn = function(self, t)
	--	self:setTalentTypeMastery("spell/earth", self:getTalentTypeMastery("spell/earth") + 0.03)
	--end,
	--on_unlearn = function(self, t)
	--	self:setTalentTypeMastery("spell/earth", self:getTalentTypeMastery("spell/earth") - 0.03)
	--end,
	info = function(self, t)
		local dur = t.getDuration(self,t)
		local dam = t.getDamage(self,t)
		return ([[Attack the target with main weapon and a shield bash, if worn, dealing %d%% damage.
		In addition, blasts enemies in a cone of radius 3 with sand, dealing %0.2f damage and blinding all those affected for %d turns.
		Damage increases with talent level and physical power.  Chance to blind increases with physical power.]]):format(100 * self:combatTalentWeaponDamage(t, 1, 1.5),dam,dur)
	end,
}

newTalent{
	name = "Earthen Shield",
	type = {"spell/stoneshield", 3},
	require = spells_req3,
	points = 5,
	mode = "sustained",
	sustain_mana = 50,
	cooldown = 25,
	no_energy = true,
	no_sustain_autoreset = true,
	tactical = { DEFEND = 2 },
	getMaxDamage = function(self, t) return self:combatTalentSpellDamage(t, 25, 500) end,
	getDisplayName = function(self, t, p)
		return ("Earthen Shield (%d/%d)"):format(self.earthen_shield_absorb,t.getMaxDamage(self,t))
	end,
	activate = function(self, t)
		game:playSoundNear(self, "talents/earth")
		self.earthen_shield_absorb = t.getMaxDamage(self,t)
		return {
			particles = self:addParticles(Particles.new("crystalline_focus", 1)),

		}
	end,
	deactivate = function(self, t, p)
		self:removeParticles(p.particles)
		return true
	end,
	do_onTakeHit = function(self, t, p, damage)
		if damage / 2 <= self.earthen_shield_absorb then
			damage = damage / 2
			self.earthen_shield_absorb = self.earthen_shield_absorb - damage
		else
			damage = damage - self.earthen_shield_absorb
			self.earthen_shield_absorb = 0
			game.logPlayer(self, "Your earthen shield crumbles under the damage!")
			-- Deactivate without loosing energy
			self:forceUseTalent(self.T_EARTHEN_SHIELD, {ignore_energy=true})
		end
		return damage
	end,
	do_act = function(self, t, p)
		local maxDamage = t.getMaxDamage(self, t)
		if (self.earthen_shield_absorb or 0) < maxDamage then
			self.earthen_shield_absorb = math.min(self.earthen_shield_absorb + (maxDamage * math.min(self:getTalentLevelRaw(t)*2,10) / 100), maxDamage)
		end
	end,
	--Increase/Decrease spell/earth mastery by 0.03
	--on_learn = function(self, t)
	--	self:setTalentTypeMastery("spell/earth", self:getTalentTypeMastery("spell/earth") + 0.03)
	--end,
	--on_unlearn = function(self, t)
	--	self:setTalentTypeMastery("spell/earth", self:getTalentTypeMastery("spell/earth") - 0.03)
	--end,
	info = function(self, t)
		return ([[%d/%d 
		Covers the user in rock and stone, negating half of all damage taken.
		Earthen Shield has a pool with a max of %0.2f hit points used to block damage, which when it reaches zero causes the talent to deactivate.
		Each turn Earthen Shield will regenerate %d%% of its maximum hit point pool.
		The hit point pool will increase with spell power.]]):format((self.earthen_shield_absorb or 0),t.getMaxDamage(self,t),t.getMaxDamage(self,t),math.min(self:getTalentLevelRaw(t)*2))
	end,
}


newTalent{
	name = "Rolling Earth",
	type = {"spell/stoneshield", 4},
	require = spells_req4,
	points = 5,
	mana = 40,
	cooldown = 18,
	tactical = { DISABLE = { stun = 3 }, CLOSEIN = 2  },
	range = 0,
	requires_target = true,
	radius = function(self, t)
		return 1 + self:getTalentLevelRaw(t)
	end,
	--target = function(self, t)
	--	return {type="cone", range=self:getTalentRange(t), friendlyfire=false, radius=self:getTalentRadius(t), talent=t}
	--end,

	target = function(self, t)
		return {type="cone", range=self:getTalentRange(t), friendlyfire=false, radius=self:getTalentRadius(t), talent=t}
	end,
	action = function(self, t)
		local tg = self:getTalentTarget(t)
		local x, y = self:getTarget(tg)
		if not x or not y then return nil end
		self:project(tg, x, y, function(px, py)
			local target = game.level.map(px, py, Map.ACTOR)
			if not target then return end
			local tx, ty = util.findFreeGrid(self.x, self.y, 5, true, {[Map.ACTOR]=true})
			if tx and ty and target:canBe("knockback") then
				target:move(tx, ty, true)
				game.logSeen(target, "%s is tossed by the earth!", target.name:capitalize())
			end
			if target:canBe("stun") then
				target:setEffect(target.EFF_STUNNED, 3 + self:getTalentLevelRaw(t), {apply_power=self:combatSpellpower()})
			else
				game.logSeen(target, "%s resists being stunned!", target.name:capitalize())
			end
		end)
		game.level.map:particleEmitter(x, y, tg.radius, "gravity_spike", {radius=tg.radius, grids=grids, tx=self.x, ty=self.y})
		game:playSoundNear(self, "talents/earth")
		return true
	end,

	--Increase/Decrease spell/earth mastery by 0.03
	--on_learn = function(self, t)
	--	self:setTalentTypeMastery("spell/earth", self:getTalentTypeMastery("spell/earth") + 0.03)
	--end,
	--on_unlearn = function(self, t)
	--	self:setTalentTypeMastery("spell/earth", self:getTalentTypeMastery("spell/earth") - 0.03)
	--end,
	info = function(self, t)
		return ([[Causes the ground to shake violently in a cone of radius %d.
		Enemies affected are tossed towards the caster and stunned for %d rounds.
		Chance to stun improves with spell power.]]):format(3 + self:getTalentLevelRaw(t), 1 + self:getTalentLevelRaw(t))
	end,
}

newTalent{
	name = "Earth Stance",
	type = {"spell/elemental-stance", 1},
	points = 1,
	mode = "sustained",
	cooldown = 0,
	elemental_stance = true,
	getStanceArmorMod = function(self,t) return math.ceil(self:combatTalentSpellDamage(t,5,40)) end,
	getStanceResistMod = function(self,t) return self:combatTalentSpellDamage(t,1,10) end,
	on_pre_use = function(self, t, silent)
		-- prevent anyone from activating other than without the "force_talent_ignore_ressources" flag
		if not self:attr("force_talent_ignore_energy") then
			if not silent then game.logPlayer(self, "You can not active Earth Stance directly.  Use Earth Strike to activate it.") end
			return false
		end
		return true
	end,
	activate = function(self,t)
		local stance_turned_off = nil
		self:attr("earth_stance", 1)
		if self:knowTalent(self.T_WATER_STANCE) and self:isTalentActive(self.T_WATER_STANCE) then
			self:forceUseTalent(self.T_WATER_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if self:knowTalent(self.T_AIR_STANCE) and self:isTalentActive(self.T_AIR_STANCE) then
			self:forceUseTalent(self.T_AIR_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if self:knowTalent(self.T_FIRE_STANCE) and self:isTalentActive(self.T_FIRE_STANCE) then
			self:forceUseTalent(self.T_FIRE_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if not stance_turned_off and self:knowTalent(self.T_ELEMENTAL_DESTRUCTION) and self:isTalentActive(self.T_ELEMENTAL_DESTRUCTION) then
			self:forceUseTalent(self.T_ELEMENTAL_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			self:forceUseTalent(self.T_ELEMENTAL_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
		end
		return {
			armor = self:addTemporaryValue("combat_armor", t.getStanceArmorMod(self,t)),
			res = self:addTemporaryValue("resists", {all=t.getStanceResistMod(self,t)})
		}
	end,
	deactivate = function(self,t,p)
		self:attr("earth_stance",-1)
		--turn Arcane Destruction on and off if on.
		if self:isTalentActive(self.T_ELEMENTAL_DESTRUCTION) then
			self:forceUseTalent(self.T_ELEMENTAL_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			self:forceUseTalent(self.T_ELEMENTAL_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
		end
		self:removeTemporaryValue("combat_armor", p.armor)
		self:removeTemporaryValue("resists", p.res)
		return true
	end,
	info = function(self, t)
		local armor = t.getStanceArmorMod(self,t)
		local res = t.getStanceResistMod(self,t)
		return ([[The user may only enter the elemental stance of Earth by using the Earth Strike talent.
		Only one stance can be active at a time.
		Earth Stance increases the user's armor by %d and resistance to all damage by %0.2f.
		Benefits increase with the Elemental Stance talent and spell power.]])
		:format(armor, res)
	end,
}
