-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009, 2010, 2011 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

newTalent{
	name = "Air Strike",
	type = {"spell/windcutter", 1},
	require = spells_req1,
	points = 5,
	random_ego = "attack",
	cooldown = 8,
	stamina = 12,
	tactical = { ATTACK = { weapon = 2 }, DISABLE = { silence = 2 } },
	getDuration = function(self,t) return math.floor(1+math.floor(self:getTalentLevel(t)/2)) end,
	radius = function(self, t)
		return math.floor(self:getTalentLevelRaw(t)/2)
	end,
	action = function(self, t)
		--figure out which weapon (or none) to use
		local weapon = nil
		local offweapon = nil
		if self:getInven("MAINHAND") then
			weapon = self:getInven("MAINHAND")[1]
			if not weapon or not weapon.combat then
				weapon = nil
			end
		end
		if self:getInven("OFFHAND") then
			offweapon = self:getInven("OFFHAND")[1]
			if not offweapon or not offweapon.combat then
				offweapon = nil
			end
			if not weapon then
				weapon = offweapon
				offweapon = nil
			end
		end
		if self:attr("disarmed") or not weapon then
			game.logSeen(self,"Attack with self.combat")
			weapon = self.combat
			offweapon = nil
		end

		local tg = {type="hit", range=self:getTalentRange(t)}
		local x, y, target = self:getTarget(tg)
		if not x or not y then return nil end
		if target then
			if core.fov.distance(self.x, self.y, x, y) > 1 then return nil end
			local speed, hit = self:attackTargetWith(target, weapon.combat, nil, self:combatTalentWeaponDamage(t, 1, 1.5))
			-- Try to silence !
			--if hit then
			--	if target:canBe("silence") then
			--		target:setEffect(target.EFF_SILENCED, t.getDuration(self,t), {apply_power=self:combatPhysicalpower()})
			--	else
			--		game.logSeen(target, "%s resists!", target.name:capitalize())
			--	end
			--end
			--If dual wielding, attack with second weapon
			if offweapon then
				local speed, hit = self:attackTargetWith(target, offweapon.combat, nil, self:combatTalentWeaponDamage(t, 1, 1.5))
			end
		end
		--Even if we miss or no target, the power must explode in a ball!
		local ball_radius = t.radius(self,t)
		local tg = {type="ball", radius=ball_radius, selffire=false, friendlyfire=false, talent=t}
		self:project(tg, x, y, DamageType.AIR_STRIKE_BALL,{dur=t.getDuration(self,t), chance=100, dam=self:combatTalentPhysicalDamage(t,10,150)})
		if ball_radius > 0 then
			local sradius = (tg.radius + 0.5) * (engine.Map.tile_w + engine.Map.tile_h) / 2
			local nb_forks = 16
			local angle_diff = 360 / nb_forks
			for i = 0, nb_forks - 1 do
				local a = math.rad(rng.range(0+i*angle_diff,angle_diff+i*angle_diff))
				local tx = x + math.floor(math.cos(a) * tg.radius)
				local ty = y + math.floor(math.sin(a) * tg.radius)
				game.level.map:particleEmitter(x, y, tg.radius, "lightning", {radius=tg.radius, grids=grids, tx=tx-x, ty=ty-y, nb_particles=25, life=8})
			end
		end
		if not self:isTalentActive(self.T_AIR_STANCE) then self:forceUseTalent(self.T_AIR_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true}) end
		return true
	end,
	--Learn and unlearn effects, add/remove air stance
	on_learn = function(self, t)
		local lev = self:getTalentLevelRaw(t)
		if lev == 1 then
			self:learnTalent(self.T_AIR_STANCE, true, nil, {no_unlearn=true})
		end
	end,
	on_unlearn = function(self, t)
		local lev = self:getTalentLevelRaw(t)
		if lev == 0 then
			self:unlearnTalent(self.T_AIR_STANCE)
		end
	end,

	info = function(self, t)
		return ([[Strike with the power of Air.  The user strikes with all their weapons, dealing %d%% damage with each and silencing the target for %d turns.
		It also creates an explosive ball of air of radius %d, centered on the target, which deals %0.2f lightning damage and silences enemies.
		User's physical power opposes target's mental save to determine silencing.
		After use, the user activates Air Stance.  This talent may be used without a target.]]):
		format(100 * self:combatTalentWeaponDamage(t, 1, 1.5), t.getDuration(self,t), t.radius(self,t), self:combatTalentPhysicalDamage(t,10,150))
	end,
}

newTalent{
	name = "Tornado Strike",
	type = {"spell/windcutter", 2},
	require = spells_req2,
	points = 5,
	random_ego = "attack",
	cooldown = 12,
	stamina = 15,
	tactical = { ATTACK = { weapon = 2 } },
	getAccuracyBuff = function(self,t) return self:combatTalentPhysicalDamage(t,1,100) end,
	getAPRBuff = function(self,t) return self:combatTalentPhysicalDamage(t,1,100) end,
	getCritBuff = function(self,t) return self:combatTalentPhysicalDamage(t,1,25) end,
	getDuration = function(self,t) return math.min(3,math.ceil(self:getTalentLevel(t)/2)) end,
	action = function(self,t)
		local tg = {type="hit", range=self:getTalentRange(t)}
		local x, y, target = self:getTarget(tg)
		if not x or not y or not target then return nil end
		if core.fov.distance(self.x, self.y, x, y) > 1 then return nil end
		self:setEffect(target.EFF_TORNADO_STRIKE_PRECISION, t.getDuration(self,t), {atk=t.getAccuracyBuff(self,t),APR=t.getAPRBuff(self,t),crit=t.getCritBuff(self,t)})
		self:attackTarget(target, nil, self:combatTalentWeaponDamage(t, 0.4, 1.0), true)
		self:attackTarget(target, nil, self:combatTalentWeaponDamage(t, 0.4, 1.0), true)
		return true
	end,
	--Increase/Decrease spell/air mastery by 0.03
	--on_learn = function(self, t)
	--	self:setTalentTypeMastery("spell/air", self:getTalentTypeMastery("spell/air") + 0.03)
	--end,
	--on_unlearn = function(self, t)
	--	self:setTalentTypeMastery("spell/air", self:getTalentTypeMastery("spell/air") - 0.03)
	--end,
	info = function(self, t)
		local acc = t.getAccuracyBuff(self,t)
		local apr = t.getAPRBuff(self,t)
		local crit = t.getCritBuff(self,t)
		local dur = t.getDuration(self,t)
		return ([[Attack with the power of a tornado, hitting the target twice with each weapon for %d%% damage.
		In addition, the user gains a %d bonus to accuracy, %d bonus to APR, and %d%% bonus to critical chance for %d turns.
		Accuracy, APR, and critical bonuses increase with physical power.]]):
		format(self:combatTalentWeaponDamage(t, 0.4, 1.0)*100, acc, apr, crit, dur)
	end,
}

newTalent{
	name = "Wind Storm",
	type = {"spell/windcutter", 3},
	mode = "sustained",
	require = spells_req3,
	points = 5,
	sustain_mana = 50,
	cooldown = 25,
	no_energy = true,
	range = function(self,t) return 1 + math.min(math.floor(self:getTalentLevelRaw(t) / 2),2) end,
	tactical = { BUFF = 3 },
	getSlow = function(self,t) return self:combatTalentSpellDamage(t,5,50)/100 end,
	activate = function(self, t)
		game:playSoundNear(self, "talents/thunderstorm")
		return {
			particle = self:addParticles(Particles.new("tempest", 1)),
		}
	end,
	do_wind_storm = function(self, t)
		if game.zone.wilderness then return end

		local grids = core.fov.circle_grids(self.x, self.y, self:getTalentRange(t), true)
		for x, yy in pairs(grids) do
			for y, _ in pairs(grids[x]) do
				local target = game.level.map(x, y, Map.ACTOR)
				if target and self:reactionToward(target) < 0 then
					-- Wind Storm
					if target:canBe("slow") and not target:hasEffect(target.EFF_WIND_STORM) then
						-- silence the apply message it if the target already has the effect
						local reapplied = false
						for eff_id, p in pairs(target.tmp) do
							local e = target.tempeffect_def[eff_id]
							if e.desc == "Wind Storm" then
								reapplied = true
							end
						end
						target:setEffect(target.EFF_WIND_SLOW, 2, {apply_power=self:combatSpellpower(), power=t.getSlow(self,t),min_dur=1, no_ct_effect=true},reapplied)
					end
				end
			end
		end
	end,
	deactivate = function(self, t, p)
		self:removeParticles(p.particle)
		return true
	end,
	--Increase/Decrease spell/air mastery by 0.03
	--on_learn = function(self, t)
	--	self:setTalentTypeMastery("spell/air", self:getTalentTypeMastery("spell/air") + 0.03)
	--end,
	--on_unlearn = function(self, t)
	--	self:setTalentTypeMastery("spell/air", self:getTalentTypeMastery("spell/air") - 0.03)
	--end,
	info = function(self, t)
		return ([[Creates a powerful windstorm around the caster, which slows all enemies within a radius of %d by %d%%.]]):
		format(self:getTalentRange(t),t.getSlow(self,t)*100)
	end,
}

newTalent{
	name = "Whirlwind Teleport",
	type = {"spell/windcutter", 4},
	require = spells_req4,
	points = 5,
	random_ego = "attack",
	cooldown = 18,
	mana = 30,
	tactical = { attack = 2, closein = 2 },
	range = function(self,t) return 3 + math.min(math.floor(self:getTalentLevel(t)),10) end,

	action = function(self,t)
		local doTeleport = true
		local tg = {type="hit", range=self:getTalentRange(t)}
		local x, y, target = self:getTarget(tg)
		if not x or not y then return nil end
		if core.fov.distance(self.x, self.y, x, y) > self:getTalentRange(t) then return nil end
		if not game.level.map.seens(x, y) then return nil end

		if self.x == x and self.y == y then
			doTeleport = false
		end

		if doTeleport then
			-- since we're using a precise teleport we'll look for a free grid first
			local tx, ty = util.findFreeGrid(x, y, 5, true, {[Map.ACTOR]=true})
			if tx and ty then
				if not self:teleportRandom(tx, ty, 0) then
					game.logSeen(self, "The spell fizzles!")
				end
			end
		end

		for i = -1, 1 do for j = -1, 1 do
			local x, y = self.x + i, self.y + j
			if (self.x ~= x or self.y ~= y) and game.level.map:isBound(x, y) and game.level.map(x, y, Map.ACTOR) then
				local target = game.level.map(x, y, Map.ACTOR)
				self:attackTarget(target, nil, self:combatTalentWeaponDamage(t, 0.25, 1.25), true)
			end
		end end
		return true
	end,
	--Increase/Decrease spell/air mastery by 0.03
	--on_learn = function(self, t)
	--	self:setTalentTypeMastery("spell/air", self:getTalentTypeMastery("spell/air") + 0.03)
	--end,
	--on_unlearn = function(self, t)
	--	self:setTalentTypeMastery("spell/air", self:getTalentTypeMastery("spell/air") - 0.03)
	--end,
	info = function(self, t)
		return ([[The user teleports up to %d spaces within line of sight and then attack each adjacent target with all weapons for %d%% damage.]]):
		format(self:getTalentRange(t),self:combatTalentWeaponDamage(t, 0.25, 1.25) * 100)
	end,
}

newTalent{
	name = "Air Stance",
	type = {"spell/elemental-stance", 1},
	points = 1,
	mode = "sustained",
	cooldown = 0,
	elemental_stance = true,
	getStanceAccuracyMod = function(self,t) return self:combatTalentSpellDamage(t,10,50) end,
	getStanceLightningMod = function(self,t) return self:combatTalentSpellDamage(t,10,50) end,
	on_pre_use = function(self, t, silent)
		-- prevent anyone from activating other than without the "force_talent_ignore_ressources" flag
		if not self:attr("force_talent_ignore_energy") then
			if not silent then game.logPlayer(self, "You can not active Air Stance directly.  Use Air Strike to activate it.") end
			return false
		end
		return true
	end,
	activate = function(self,t)
		local stance_turned_off = nil
		self:attr("air_stance", 1)
		if self:knowTalent(self.T_WATER_STANCE) and self:isTalentActive(self.T_WATER_STANCE) then
			self:forceUseTalent(self.T_WATER_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if self:knowTalent(self.T_EARTH_STANCE) and self:isTalentActive(self.T_EARTH_STANCE) then
			self:forceUseTalent(self.T_EARTH_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if self:knowTalent(self.T_FIRE_STANCE) and self:isTalentActive(self.T_FIRE_STANCE) then
			self:forceUseTalent(self.T_FIRE_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if not stance_turned_off and self:knowTalent(self.T_ELEMENTAL_DESTRUCTION) and self:isTalentActive(self.T_ELEMENTAL_DESTRUCTION) then
			self:forceUseTalent(self.T_ELEMENTAL_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			self:forceUseTalent(self.T_ELEMENTAL_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
		end
		return {
			atk = self:addTemporaryValue("combat_atk", t.getStanceAccuracyMod(self,t)),
			dam = self:addTemporaryValue("melee_project", {[DamageType.LIGHTNING] = t.getStanceLightningMod(self, t)}),
		}
	end,
	deactivate = function(self,t,p)
		self:attr("air_stance",-1)
		--turn Arcane Destruction on and off if on.
		if self:isTalentActive(self.T_ELEMENTAL_DESTRUCTION) then
			self:forceUseTalent(self.T_ELEMENTAL_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			self:forceUseTalent(self.T_ELEMENTAL_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
		end
		self:removeTemporaryValue("combat_atk", p.atk)
		self:removeTemporaryValue("melee_project", p.dam)
		return true
	end,
	info = function(self, t)
		local atk = t.getStanceAccuracyMod(self,t)
		local dam = t.getStanceLightningMod(self,t)
		return ([[The user may only enter the elemental stance of Air by using the Air Strike talent.
		Only one stance can be active at a time.
		Air Stance increases the user's accuracy by %d and deals %0.2f lightning damage with each weapon strike.
		Benefits increase with the Elemental Stance talent and spell power.]])
		:format(atk, dam)
	end,
}
