-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009, 2010, 2011 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org


newTalent{
	name = "Arcane Stamina",
	type = {"technique/arcane-veteran", 1},
	require = techs_req1,
	no_energy = true,
	sustain_mana = 20,
	mode = "sustained",
	tactical = { BUFF = 2 },
	points = 5,
	cooldown = 5,
	get_stamina_regen = function(self,t)
		local sustain_count = 0

		for tid, act in pairs(self.sustain_talents) do
			sustain_count = sustain_count + 1
		end
		return (sustain_count * self:getTalentLevel(t) * 0.1)
	end,
	activate = function(self, t)
		local sustain_count = 1

		for tid, act in pairs(self.sustain_talents) do
			sustain_count = sustain_count + 1
		end
		self.stamina_regen = self.stamina_regen + (sustain_count  * self:getTalentLevel(t) * 0.1)
		return {
			stam = self:addTemporaryValue("arcane_stamina_mult", self:getTalentLevel(t) * 0.1)
		}
	end,
	deactivate = function(self, t, p)
		local sustain_count = 0

		for tid, act in pairs(self.sustain_talents) do
			sustain_count = sustain_count + 1
		end
		self.stamina_regen = self.stamina_regen - (sustain_count * self:attr("arcane_stamina_mult"))
		self:removeTemporaryValue("arcane_stamina_mult", p.stam)
		return true
	end,
	info = function(self, t)
		return ([[The user magically taps into their sustained powers, siphoning off a portion of the energy to restore the user's stamina.
		Increases stamina regen by %0.2f per active sustain.]]):format(self:getTalentLevel(t)*0.1)
	end,
}

newTalent{
	name = "Mana Focus",
	type = {"technique/arcane-veteran", 2},
	require = techs_req2,
	mode = "sustained",
	sustain_stamina = 40,
	no_energy = true,
	cooldown = 5,
	points = 5,
	tactical = { BUFF = 2 },
	activate = function(self, t)
		return {
			regen = self:addTemporaryValue("mana_regen", self:getTalentLevel(t) / 10),
			man = self:addTemporaryValue("mana_regen_on_hit", (0.4 * self:getTalentLevel(t))),
		}
	end,
	deactivate = function(self, t, p)
		self:removeTemporaryValue("mana_regen", p.regen)
		self:removeTemporaryValue("mana_regen_on_hit", p.man)
		return true
	end,
	info = function(self, t)
		return ([[By entering a combat trance, the user is able to convert the focus and discipline needed for martial combat into mana.  The user gains %0.2f mana for every successful hit, as well as restoring %0.2f mana each turn.]]):format(0.4 *self:getTalentLevel(t),self:getTalentLevel(t)/10)
	end,
}

newTalent{
	name = "Arcane Armor",
	type = {"technique/arcane-veteran", 3},
	require = techs_req3,
	mode = "sustained",
	sustain_mana = 40,
	no_energy = true,
	cooldown = 5,
	points = 5,
	tactical = { BUFF = 2 },
	activate = function(self, t)
		local power = self:getTalentLevel(t) * 3
		return {
			fatigue = self:addTemporaryValue("fatigue", -power),
		}
	end,
	deactivate = function(self, t, p)
		self:removeTemporaryValue("fatigue",p.fatigue)
		return true
	end,
	info = function(self, t)
		local power = self:getTalentLevel(t) * 3
		return ([[Enchants the user's armor, making it lighter and stronger.  Reduces fatigue by %d%% and increases body armor protection by %d%%.]]):format(power,self:getTalentLevel(t)*30)
	end,
}

newTalent{
	name = "Martial Magic", -- no cost; it's main purpose is to give the player an alternative means of using mana/stamina based talents
	type = {"technique/arcane-veteran", 4},
	require = techs_req4,
	points = 5,
	cooldown = 24,
	tactical = { MANA = 1, STAMINA = 1, BUFF = 1 },
	getDuration = function(self, t) return 2 + math.ceil(self:getTalentLevel(t)) end,
	no_energy = true,
	action = function(self, t)
		self:setEffect(self.EFF_MARTIAL_MAGIC, t.getDuration(self, t), {power = 10})
		return true
	end,
	info = function(self, t)
		local duration = t.getDuration(self, t)
		return ([[The user enters a mystical martial form for %0.2f turns which allows them to utilize their stamina as mana or their mana as stamina.  The user's mana and stamina sustains will not drop unless both their mana and stamina reach zero. This talent will automatically trigger if the user's mana or stamina drops to zero as long as it is not cooling down.   
		Using this talent does not take a turn.]]):
		format(duration)
	end,
}



