-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009, 2010, 2011 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

local damDesc = Talents.main_env.damDesc

newTalent{
	name = "Spellsword Combat",
	type = {"technique/elemental-combat", 1},
	mode = "sustained",
	points = 5,
	require = techs_req1,
	sustain_stamina = 20,
	sustain_mana = 0,
	no_energy = true,
	cooldown = 5,
	tactical = { BUFF = 2 },
	range = 10,
	direct_hit = true,
	reflectable = true,
	is_spell = true,
	target = function(self, t)
		return {type="beam", range=self:getTalentRange(t), talent=t}
	end,
	--effective combat Talent spell damage using the whole tree to deterimine talent level
	modifiedCombatTalentSpellDamage = function(self,t,eff_talent_level,base,max,spellpower_override)
		-- Compute at "max"
		local mod = max / ((base + 100) * ((math.sqrt(5) - 1) * 0.8 + 1))
		-- Compute real
		return self:rescaleDamage((base + (spellpower_override or self:combatSpellpower())) * ((math.sqrt(eff_talent_level) - 1) * 0.8 + 1) * mod)
	end,
	getTargetCount = function(self, t) return math.min(1 + self:getTalentLevelRaw(t),6) end,
	getConeRadius = function(self,t) return math.min(2 + math.ceil(self:getTalentLevelRaw(t)),7) end,
	getBallRadius = function(self,t) return math.min(math.ceil(self:getTalentLevelRaw(t) / 2.0),3) end,
	getBeamRange = function(self,t) return math.min(5 + self:getTalentLevelRaw(t),10) end,
	getDamage = function(self,t,eff_talent_level) return t.modifiedCombatTalentSpellDamage(self,t,1 + eff_talent_level, 25, 250) end,
	get_mana_per_trigger = function(self,t)
	  return 10 * (100 + 2 * self:combatFatigue()) / 100
	end,
	do_trigger = function(self, t, target)
		--make sure we're not blasting ourselves
		if self.x == target.x and self.y == target.y then return nil end
		--Check dex chance
		
		local chance = 25 + self:getTalentLevel(t) * (1 + self:getDex(9, true))
		if self:hasDualWeapon() then chance = chance / 2 end
		
		if rng.percent(chance) then
			local mana = self:getMana() - 1
			local mana_per_trigger = t.get_mana_per_trigger(self,t)
			if mana > mana_per_trigger then
				--Air stance -> LIGHTNING!
				if self:isTalentActive(self.T_AIR_STANCE) then
					local eff_talent_level = (self:getTalentLevel(self.T_AIR_STRIKE) + self:getTalentLevel(self.T_TORNADO_STRIKE) + self:getTalentLevel(self.T_WIND_STORM) + self:getTalentLevel(self.T_WHIRLWIND_TELEPORT)) / 4

					local tg = {type="bolt", range=self:getTalentRange(t),friendlyfire=false, talent=t}
					local fx = target.x
					local fy = target.y

					local nb = t.getTargetCount(self, t)
					local affected = {}
					local first = nil

					self:project(tg, fx, fy, function(dx, dy)
						print("[Chain lightning] targetting", fx, fy, "from", self.x, self.y)
						local actor = game.level.map(dx, dy, Map.ACTOR)
						if actor and not affected[actor] then
							ignored = false
							affected[actor] = true
							first = actor

							print("[Chain lightning] looking for more targets", nb, " at ", dx, dy, "radius ", 10, "from", actor.name)
							self:project({type="ball", selffire=false,friendlyfire=false, x=dx, y=dy, radius=10, range=0}, dx, dy, function(bx, by)
								local actor = game.level.map(bx, by, Map.ACTOR)
								if actor and not affected[actor] and self:reactionToward(actor) < 0 then
									print("[Chain lightning] found possible actor", actor.name, bx, by, "distance", core.fov.distance(dx, dy, bx, by))
									affected[actor] = true
								end
							end)
							return true
						end
					end)

					if not first then return end
					local targets = { first }
					affected[first] = nil
					local possible_targets = table.listify(affected)
					print("[Chain lightning] Found targets:", #possible_targets)
					for i = 2, nb do
						if #possible_targets == 0 then break end
						local act = rng.tableRemove(possible_targets)
						targets[#targets+1] = act[1]
					end

					local sx, sy = self.x, self.y
					for i, actor in ipairs(targets) do
						local tgr = {type="beam", range=self:getTalentRange(t), selffire=false,friendlyfire=false, talent=t, x=sx, y=sy}
						print("[Chain lightning] jumping from", sx, sy, "to", actor.x, actor.y)
						self:project(tgr, actor.x, actor.y, DamageType.LIGHTNING, self:spellCrit(t.getDamage(self,t,eff_talent_level)))
						game.level.map:particleEmitter(sx, sy, math.max(math.abs(actor.x-sx), math.abs(actor.y-sy)), "lightning", {tx=actor.x-sx, ty=actor.y-sy, nb_particles=150, life=6})
						sx, sy = actor.x, actor.y
					end

					game:playSoundNear(self, "talents/lightning")
					self:incMana(-mana_per_trigger)
				end
				-- Earth Stance -> PHYSICAL!
				if self:isTalentActive(self.T_EARTH_STANCE) then
					local tg = {type="ball", range=1, radius=t.getBallRadius(self,t), selffire=false,friendlyfire=false, talent=t}
					local eff_talent_level = (self:getTalentLevel(self.T_EARTH_STRIKE) + self:getTalentLevel(self.T_SANDBLAST) + self:getTalentLevel(self.T_EARTHEN_SHIELD) + self:getTalentLevel(self.T_ROLLING_EARTH)) / 4
					self:project(tg, target.x, target.y, DamageType.PHYSICAL, self:spellCrit(t.getDamage(self, t,eff_talent_level)))
					local _ _, x, y = self:canProject(tg, target.x, target.y)
					game.level.map:particleEmitter(target.x, target.y, tg.radius, "ball_earth", {radius=tg.radius, tx=target.x, ty=target.y})
					game:playSoundNear(self, "talents/earth")
					self:incMana(-mana_per_trigger)
				end
				-- Fire stance -> FIRE!
				if self:isTalentActive(self.T_FIRE_STANCE) then
					local tg = {type="cone", range=0, radius=t.getConeRadius(self,t), selffire=false,friendlyfire=false, talent=t}
					local eff_talent_level = (self:getTalentLevel(self.T_FIRE_STRIKE) + self:getTalentLevel(self.T_INFERNO_CLEAVE) + self:getTalentLevel(self.T_AURA_OF_FIRE) + self:getTalentLevel(self.T_MARK_OF_FIRE)) / 4
					self:project(tg, target.x, target.y, DamageType.FIRE, self:spellCrit(t.getDamage(self, t,eff_talent_level)))
					local _ _, x, y = self:canProject(tg, target.x, target.y)
					game.level.map:particleEmitter(self.x, self.y, tg.radius, "breath_fire", {radius=tg.radius, tx=target.x-self.x, ty=target.y-self.y})
					game:playSoundNear(self, "talents/fire")
					self:incMana(-mana_per_trigger)
				end
				-- Water Stance -> COLD
				if self:isTalentActive(self.T_WATER_STANCE) then
					local l = self:lineFOV(target.x, target.y)
					l:set_corner_block()
					local lx, ly, is_corner_blocked = l:step(true)
					local x, y = lx, ly
					-- Check for terrain and friendly actors
					while lx and ly and not is_corner_blocked and core.fov.distance(self.x, self.y, lx, ly) <= t.getBeamRange(self,t) do
						local actor = game.level.map(lx, ly, engine.Map.ACTOR)
						if actor and (self:reactionToward(actor) >= 0) then
							break
						elseif game.level.map:checkEntity(lx, ly, engine.Map.TERRAIN, "block_move") then
							x, y = lx, ly
							break
						end
						x, y = lx, ly
						lx, ly = l:step(true)
					end
					--Do the Beam!
					local tg = {type="beam", range=t.getBeamRange(self,t), talent=t}
					local eff_talent_level = (self:getTalentLevel(self.T_WATER_STRIKE) + self:getTalentLevel(self.T_HEALING_WATERS) + self:getTalentLevel(self.T_TIDAL_RUSH) + self:getTalentLevel(self.T_DRAIN_WATER_OF_LIFE)) / 4
					self:project(tg, x, y, DamageType.COLD, self:spellCrit(t.getDamage(self,t,eff_talent_level)))
					local _ _, x, y = self:canProject(tg, x, y)
					game.level.map:particleEmitter(self.x, self.y, tg.radius, "icebeam", {tx=x-self.x, ty=y-self.y})
					game:playSoundNear(self, "talents/water")
					self:incMana(-mana_per_trigger)
				end
			end
		end
	end,
	activate = function(self, t)
		return {}
	end,
	deactivate = function(self, t, p)
		return true
	end,
	info = function(self, t)
		mana_per_trigger = t.get_mana_per_trigger(self,t)
		local water_eff_talent_level = (self:getTalentLevel(self.T_WATER_STRIKE) + self:getTalentLevel(self.T_HEALING_WATERS) + self:getTalentLevel(self.T_TIDAL_RUSH) + self:getTalentLevel(self.T_DRAIN_WATER_OF_LIFE)) / 4
		local fire_eff_talent_level = (self:getTalentLevel(self.T_FIRE_STRIKE) + self:getTalentLevel(self.T_INFERNO_CLEAVE) + self:getTalentLevel(self.T_AURA_OF_FIRE) + self:getTalentLevel(self.T_MARK_OF_FIRE)) / 4
		local earth_eff_talent_level = (self:getTalentLevel(self.T_EARTH_STRIKE) + self:getTalentLevel(self.T_SANDBLAST) + self:getTalentLevel(self.T_EARTHEN_SHIELD) + self:getTalentLevel(self.T_ROLLING_EARTH)) / 4
		local air_eff_talent_level = (self:getTalentLevel(self.T_AIR_STRIKE) + self:getTalentLevel(self.T_TORNADO_STRIKE) + self:getTalentLevel(self.T_WIND_STORM) + self:getTalentLevel(self.T_WHIRLWIND_TELEPORT)) / 4
		return ([[Allows the user's mainhand melee weapon to focus elemental spells, granting %d%% chance per melee attack to deliver an effect depedent on stance.  When using two weapons, the chance is halved for each weapon.

		Air: Cast chain lightning hitting up to %d targets for %0.2f damage.

		Earth: Summon an explosive ball of stone hitting a radius %d for %0.2f damage.

		Fire: Create a cone of flame hitting a radius %d for %0.2f damage.

		Water: Blast a cold beam up to %d long hitting for %0.2f damage.

		Each time a spell is channeled it uses %0.2f mana.  Trigger chance increases with talent points and dexterity.
		Damage increases with talents spent in the corresponding stance's elemental tree and spellpower.]]):
		format(25 + self:getTalentLevel(t) * (1 + self:getDex(9, true)),
		t.getTargetCount(self,t), damDesc(self, DamageType.LIGHTNING,t.getDamage(self,t,air_eff_talent_level)),
		t.getBallRadius(self,t), damDesc(self, DamageType.PHYSICAL,t.getDamage(self,t,earth_eff_talent_level)),
		t.getConeRadius(self,t), damDesc(self, DamageType.FIRE,t.getDamage(self,t,fire_eff_talent_level)),
		t.getBeamRange(self,t), damDesc(self, DamageType.COLD,t.getDamage(self,t,water_eff_talent_level)),mana_per_trigger)
	end,
}

newTalent{
	name = "Elemental Stance",
	type = {"technique/elemental-combat", 2},
	mode = "passive",
	points = 5,
	require = techs_req2,
	modifiedCombatTalentSpellDamage = function(self,t,eff_talent_level,base,max,spellpower_override)
		-- Compute at "max"
		local mod = max / ((base + 100) * ((math.sqrt(5) - 1) * 0.8 + 1))
		-- Compute real
		return self:rescaleDamage((base + (spellpower_override or self:combatSpellpower())) * ((math.sqrt(eff_talent_level) - 1) * 0.8 + 1) * mod)
	end,
	--Increase/Decrease spell/fire mastery by talent points
	on_learn = function(self, t)
		self:setTalentTypeMastery("spell/elemental-stance", 1 + self:getTalentLevelRaw(t))
	end,
	on_unlearn = function(self, t)
		self:setTalentTypeMastery("spell/elemental-stance", 1 + self:getTalentLevelRaw(t))
	end,
	info = function(self, t)
		local air_acc = t.modifiedCombatTalentSpellDamage(self,t,self:getTalentLevel(t) + 1,10,50)
		local air_dmg = t.modifiedCombatTalentSpellDamage(self,t,self:getTalentLevel(t) + 1,10,50)
		local earth_arm = t.modifiedCombatTalentSpellDamage(self,t,self:getTalentLevel(t) + 1,5,40)
		local earth_res = t.modifiedCombatTalentSpellDamage(self,t,self:getTalentLevel(t) + 1,1,10)
		local fire_phys = t.modifiedCombatTalentSpellDamage(self,t,self:getTalentLevel(t) + 1,10,50)
		local fire_crit = t.modifiedCombatTalentSpellDamage(self,t,self:getTalentLevel(t) + 1,2,20)
		local water_healmod = t.modifiedCombatTalentSpellDamage(self,t,self:getTalentLevel(t) + 1,5,40)/100
		local water_healhit = t.modifiedCombatTalentSpellDamage(self,t,self:getTalentLevel(t) + 1,2,20)
		return ([[The user improves the effects of their Elemental Stances, increasing their effective talent mastery level by %d.

		Air Stance increases accuracy by %d and deals %0.2f lightning damage per melee hit.

		Earth Stance increase armor by %d and resistance to all by %0.2f.

		Fire Stance increases physical power by %d and physical critical change by %0.2f.

		Water Stance increases healing by %d%% and heals %0.2f per melee hit.

		Each stance is activated by its corresponding elemental strike.]]):format(self:getTalentLevelRaw(t),air_acc,air_dmg,earth_arm,earth_res,fire_phys,fire_crit, water_healmod * 100, water_healhit)
	end,
}

newTalent{
	name = "Arcane Prowess",
	type = {"technique/elemental-combat", 3},
	mode = "passive",
	points = 5,
	require = techs_req3,
	info = function(self, t)
		return ([[The user gains a bonus to spell power equal to %d%% of their strength plus dexterity.]]):format(5 + (self:getTalentLevel(t) * 5))
	end,
}

newTalent{
	name = "Elemental Destruction",
	type = {"technique/elemental-combat", 4},
	mode = "sustained",
	points = 5,
	require = techs_req4,
	sustain_mana = 50,
	no_energy = true,
	cooldown = 5,
	tactical = { BUFF = 2 },
	getDamageIncrease = function(self, t) return self:getTalentLevelRaw(t) * 2 end,
	getResistPenalty = function(self, t) return self:getTalentLevelRaw(t) * 10 end,
	activate = function(self,t)
		if self:attr("air_stance") then
			return {
				elemdam = self:addTemporaryValue("inc_damage", {[DamageType.LIGHTNING] = t.getDamageIncrease(self, t)}),
				resist = self:addTemporaryValue("resists_pen", {[DamageType.LIGHTNING] = t.getResistPenalty(self, t)}),
			}
		end
		if self:attr("earth_stance") then
			return {
				elemdam = self:addTemporaryValue("inc_damage", {[DamageType.PHYSICAL] = t.getDamageIncrease(self, t)}),
				resist = self:addTemporaryValue("resists_pen", {[DamageType.PHYSICAL] = t.getResistPenalty(self, t)}),
			}
		end
		if self:attr("fire_stance") then
			return {
				elemdam = self:addTemporaryValue("inc_damage", {[DamageType.FIRE] = t.getDamageIncrease(self, t)}),
				resist = self:addTemporaryValue("resists_pen", {[DamageType.FIRE] = t.getResistPenalty(self, t)}),
			}
		end
		if self:attr("water_stance") then
			return {
				elemdam = self:addTemporaryValue("inc_damage", {[DamageType.COLD] = t.getDamageIncrease(self, t)}),
				resist = self:addTemporaryValue("resists_pen", {[DamageType.COLD] = t.getResistPenalty(self, t)}),
			}
		end
		return {
		elemdam = self:addTemporaryValue("inc_damage",{[DamageType.PHYSICAL] = 0}),
		resist = self:addTemporaryValue("resists_pen", {[DamageType.PHYSICAL] = 0}),
		}
	end,
	deactivate = function(self,t,p)
		self:removeTemporaryValue("inc_damage", p.elemdam)
		self:removeTemporaryValue("resists_pen", p.resist)
		return true
	end,
	info = function(self, t)
		return ([[Raw magical damage channels through the caster's weapon, changing the weapon into pure Lightning (Air Stance), Stone (Earth Stance), Fire (Fire Stance), or Cold (Water Stance).
		The magic increases all corresponding elemental damage by %d%% and allows the user to ignore %d%% of the target's corresponding resistance.]]):
		format(self:getTalentLevelRaw(t)*2,self:getTalentLevelRaw(t)*10)
	end,
}

