local Map = require "engine.Map"

newTalent{
	name = "Blood Master",
	type = {"technique/executioner", 1},
	mode = "passive",
	points = 5,
	require = barb_req_high1,
	mode = "passive",
	do_cut = function(self, t, target, dam)
		if target:canBe("cut") and rng.percent(10+(self:getTalentLevel(t)*10)) then
			dam = dam * self:combatTalentWeaponDamage(t, 0.15, 0.35)
			target:setEffect(target.EFF_CUT, 5, {src=self, power=(dam / 10)})
		end
	end,
	info = function(self, t)
		return ([[You become more aware of bleeding points on your enemies. Each strike now has a chance to cause bleeding damage over 5 turns.
		
		%d%% Chance per hit
		%d%% Attack Damage -> Bleed over 5 turns]]):
		format(10+(self:getTalentLevel(t)*10),100 * self:combatTalentWeaponDamage(t, 0.15, 0.35))
	end,
}

newTalent{
	name = "Scent of Death",
	type = {"technique/executioner", 2},
	points = 5,
	random_ego = "attack",
	cooldown = 25,
	stamina = 12,
	require = barb_req_high2,
	no_energy = true,
	tactical = { ATTACK = 4 },
	action = function(self, t)
		self:setEffect(self.EFF_ATTACK, 1 + self:getTalentLevel(t), {power=100})
		return true
	end,
	info = function(self, t)
		return ([[You might not be able to see it, but you sure can smell that it wants to die.
		
		+100 Accuracy for %d turns
		
		Ignores invisibility and stealth.]]):
		format(1 + self:getTalentLevel(t))
	end,
}

newTalent{
	name = "Call of Zigur",
	type = {"technique/executioner", 3},
	require = barb_req_high3,
	points = 5,
	random_ego = "attack",
	cooldown = 14,
	stamina = 20,
	tactical = { CLOSEIN = 2 },
	range = 0,
	radius = function(self, t)
		return 2 + self:getTalentLevel(t)
	end,
	target = function(self, t)
		return {type="ball", range=self:getTalentRange(t), friendlyfire=false, radius=self:getTalentRadius(t), talent=t}
	end,
	action = function(self, t)
		local tg = self:getTalentTarget(t)
		self:project(tg, self.x, self.y, function(px, py)
			local target = game.level.map(px, py, Map.ACTOR)
			if not target then return end
			local tx, ty = util.findFreeGrid(self.x, self.y, 5, true, {[Map.ACTOR]=true})
			if tx and ty and target:canBe("teleport") then
				target:move(tx, ty, true)
				game.logSeen(target, "%s is called to battle!", target.name:capitalize())
			end
		end)
		return true
	end,
	info = function(self, t)
		return ([[Zigur warlords used to speak of this forgotten battle cry that was used sparingly by roaming barbarian hunters.
		
		Call all foes in a radius of %d to melee range instantly.]]):format(2+self:getTalentLevel(t))
	end,
}

newTalent{
	name = "Execution",
	type = {"technique/executioner", 4},
	require = barb_req_high4,
	points = 5,
	cooldown = 18,
	stamina = 35,
	requires_target = true,
	tactical = { ATTACK = { weapon = 2, cut = 4 }, DISABLE = 2 },
	action = function(self, t)
		local tg = {type="hit", range=self:getTalentRange(t)}
		local x, y, target = self:getTarget(tg)
		if not x or not y or not target then return nil end
		if core.fov.distance(self.x, self.y, x, y) > 1 then return nil end

		local hit = self:attackTarget(target, nil, self:combatTalentWeaponDamage(t, 2, 3.5), true)
		if hit then
			if target:canBe("cut") then
				local sw = self:getInven("MAINHAND")
				if sw then
					sw = sw[1] and sw[1].combat
				end
				sw = sw or self.combat
				local dam = self:combatDamage(sw)
				local damrange = self:combatDamageRange(sw)
				dam = rng.range(dam, dam * damrange)
				dam = dam * self:combatTalentWeaponDamage(t, 2.5, 4)

				target:setEffect(target.EFF_DEEP_WOUND, 4, {src=self, heal_factor=self:getTalentLevel(t) * 10, power=dam / 4, apply_power=self:combatAttack()})
			end
		end
		return true
	end,
	info = function(self, t)
		return ([[You put all your strength into one huge attack at your enemy's most vital artery, hoping to bleed him out like a gutted pig.
		
		%d%% Weapon damage.
		%d%% Bleeding damage over 4 turns
		
		%d%% Healing Reduction]]):
		format(100 * self:combatTalentWeaponDamage(t, 1, 1.7), 100 * self:combatTalentWeaponDamage(t, 2, 3.2), self:getTalentLevel(t) * 10)
	end,
}
