newTalent{
	name = "Brute",
	type = {"technique/gladiator", 1},
	mode = "passive",
	points = 5,
	require = barb_req1,
   on_learn = function(self, t)
      if self:getTalentLevelRaw(t) == 1 then
         self:attr("allow_any_dual_weapons", 1)
      end
   end,
   on_unlearn = function(self, t)
      if not self:knowTalent(t) then
         self:attr("allow_any_dual_weapons", - 1)
      end
   end,
	info = function(self, t)
		return ([[Your arms are strong enough to wield two full size one handed weapons easily.
		
		%d%% Off-hand Weapon Damage]]):
		format(100 / (2 - (math.min(self:getTalentLevel(t), 8) / 9)), 100 * self:combatTalentWeaponDamage(t, 0.5, 1.1))
	end,
}

newTalent{
	name = "Charge",
	type = {"technique/gladiator", 2},
	message = "@Source@ charges out!",
	require = barb_req2,
	points = 5,
	random_ego = "attack",
	stamina = 20,
	cooldown = 13,
	tactical = { ATTACK = { weapon = 1.2, stun = 1.3 }, CLOSEIN = 3 },
	requires_target = true,
	range = function(self, t) return math.floor(5 + self:getTalentLevelRaw(t)) end,
	on_pre_use = function(self, t)
		if self:attr("never_move") then return false end
		return true
	end,
	action = function(self, t)
		local tg = {type="hit", range=self:getTalentRange(t)}
		local x, y, target = self:getTarget(tg)
		if not x or not y or not target then return nil end
		if core.fov.distance(self.x, self.y, x, y) > self:getTalentRange(t) then return nil end

		local block_actor = function(_, bx, by) return game.level.map:checkEntity(bx, by, Map.TERRAIN, "block_move", self) end
		local l = self:lineFOV(x, y, block_actor)
		local lx, ly, is_corner_blocked = l:step()
		if is_corner_blocked or game.level.map:checkAllEntities(lx, ly, "block_move", self) then
			game.logPlayer(self, "You are too close to build up momentum!")
			return
		end
		local tx, ty = lx, ly
		lx, ly, is_corner_blocked = l:step()
		while lx and ly do
			if is_corner_blocked or game.level.map:checkAllEntities(lx, ly, "block_move", self) then break end
			tx, ty = lx, ly
			lx, ly, is_corner_blocked = l:step()
		end

		local ox, oy = self.x, self.y
		self:move(tx, ty, true)
		if config.settings.tome.smooth_move > 0 then
			self:resetMoveAnim()
			self:setMoveAnim(ox, oy, 8, 5)
		end

		-- Attack ?
		if core.fov.distance(self.x, self.y, x, y) == 1 then
			if self:knowTalent(self.T_STEAMROLLER) then
				target:setEffect(target.EFF_STEAMROLLER, 2, {src=self})
				self:setEffect(self.EFF_STEAMROLLER_USER, 2, {buff=20})
			end

			if self:attackTarget(target, nil, 1.2, true) and target:canBe("stun") then
				-- Stun, no save
				target:setEffect(target.EFF_STUNNED, 3, {})
			end
		end

		
		return true
	end,
	info = function(self, t)
		return ([[Charges at your target with incredible speed. You must charge from at least 2 tiles away.
		
		130% Weapon Damage
		Stuns enemy for 3 turns
		
		Range increases with more points.
		
		* Steamroller prodigy also effects this talent.]])
	end,
}


newTalent{
	name = "Endurance",
	type = {"technique/gladiator", 3},
	mode = "passive",
	points = 5,
	require = barb_req3,
	on_learn = function(self, t)
		self.stamina_regen = self.stamina_regen + 0.5
		self.life_regen = self.life_regen + 1
		self:attr("max_encumber", 20)

	end,
	on_unlearn = function(self, t)
		self.stamina_regen = self.stamina_regen - 0.5
		self.life_regen = self.life_regen - 1
		self:attr("max_encumber", -20)

	end,
	info = function(self, t)
		return ([[Your vast experience in the Arena has taught you to recover quickly in the mist of battle. You also know the true value of worthless junk, and can carry more of it.
		
		+%0.2f Stamina / Turn
		+%0.2f Life / Turn
		
		+%d Max. Encumberance]]):format(self:getTalentLevelRaw(t) / 2, self:getTalentLevelRaw(t), self:getTalentLevelRaw(t) * 20)
	end,
}

newTalent{
	name = "Guard",
	type = {"technique/gladiator", 4},
	points = 5,
	require = barb_req4,
	mode = "passive",
	on_learn = function(self, t)
		self.combat_def = self.combat_def + 2
		self.combat_physresist = self.combat_physresist + 5
		self.combat_armor = self.combat_armor + 3
	end,
	on_unlearn = function(self, t)
		self.combat_def = self.combat_def - 2
		self.combat_physresist = self.combat_physresist - 5
		self.combat_armor = self.combat_armor - 3
	end,
	info = function(self, t)

		return ([[Your body is hardened from constant beatings, and you are learning to anticipate incoming attacks.
		
		+%d Defense
		+%d Physical Save
		+%d Armour Value]]):
		format(self:getTalentLevelRaw(t) * 2, self:getTalentLevelRaw(t) * 5, self:getTalentLevelRaw(t) * 3)
	end,
}

