newTalent{
	name = "Weapon Stance",
	type = {"technique/martial-mastery", 1},
	mode = "passive",
	points = 5,
	require = dex_req_high1,
	on_learn = function(self, t)
		local lev = self:getTalentLevelRaw(t)
		if lev == 1 then
			self:learnTalent(self.T_DRAGON_STANCE, true, nil, {no_unlearn=true})
			self:learnTalent(self.T_BEAR_STANCE, true, nil, {no_unlearn=true})
			self:learnTalent(self.T_FALCON_STANCE, true, nil, {no_unlearn=true})
		end
	end,
	on_unlearn = function(self, t)
		local lev = self:getTalentLevelRaw(t)
		if lev == 0 then
			self:unlearnTalent(self.T_DRAGON_STANCE)
			self:unlearnTalent(self.T_BEAR_STANCE)
			self:unlearnTalent(self.T_FALCON_STANCE)
		end
	end,
	info = function(self, t)
		local tl = self:getTalentLevel(t)
		return ([[The user learns 3 stances, unique sustains usuable one at a time.
		
		Dragon Stance: Increases physical power by %d and physical crit chance by %d%%. Decreases accuracy by %d and defense by %d.
		
		Bear Stance: Increases defense by %d and all saves by %d.  Decreases physical crit chance by %d%% and physical attack speed by %d%%.
		
		Falcon Stance: Increases accuracy by %d and physical attack speed by %d%%.  Decreases physical power by %d and all saves by %d.]]):
		format(tl*10,tl*5,-tl*5,-tl*5,tl*10,tl*6,-tl*2.5,-tl*2,tl*10,tl*4,-tl*5,-tl*3)
	end,
}

newTalent{
	name = "Combat Grip",
	type = {"technique/martial-mastery", 2},
	mode = "passive",
	points = 5,
	require = dex_req_high2,
	on_learn = function(self, t)
		self.disarm_immune = (self.disarm_immune or 0) + .2
	end,
	on_unlearn = function(self, t)
		self.disarm_immune = (self.disarm_immune or 0) - .2
	end,
	do_trigger = function(self, t, target)
		--make sure we're not blasting ourselves
		if self.x == target.x and self.y == target.y then return nil end
		--Check dex chance
		
		local chance = 2 * self:getTalentLevel(t)
		if self:hasDualWeapon() then chance = chance / 2 end
		
		if rng.percent(chance) and target:canBe("disarm") then
			target:setEffect(target.EFF_DISARMED, 2 , {apply_power=self:combatAttack()})
			target:crossTierEffect(target.EFF_DISARMED, self:combatAttack())
		end
	end,
	info = function(self, t)
		return ([[The user has learned advanced combat techniques to keep their weapon in their hand, and their enemy's weapon out of theirs.
		The user gains %d%% disarm resistance, and has a %d%% chance to disarm an opponent for 2 turns on hit.  This chance is halved when dualwielding.
		The chance for the disarm to succeed increases with accuracy.]]):format(20*self:getTalentLevelRaw(t),2*self:getTalentLevel(t))
	end,
}

newTalent{
	name = "Parry",
	type = {"technique/martial-mastery", 3},
	cooldown = 8,
	points = 5,
	require = dex_req_high3,
	range = 1,
	tactical = { ATTACK = 3, DEFEND = 3 },
	on_pre_use = function(self, t, silent) if not self:getInven("MAINHAND") then if not silent then game.logPlayer(self, "You require a mainhand weapon to use this talent.") end return false end return true end,
	getProperties = function(self, t)
		local p = {
			sp = false,
			ref = false,
			br = false,
		}
		return p
	end,
	getBlockValue = function(self, t)
		local val = 0
		if not self:getInven("MAINHAND") then return val end
		local weapon = self:getInven("MAINHAND")[1]
		if weapon then val = val + (weapon.combat and weapon.combat.dam or 0) * self:getTalentLevel(t) / 1.5 end
		return val
	end,
	getBlockedTypes = function(self, t)
		local bt = {[DamageType.PHYSICAL]=true}
		local weapon = self:getInven("MAINHAND") and self:getInven("MAINHAND")[1]
		weapon = weapon and weapon.combat

		if self:knowTalent(self.T_ELEMENTAL_DESTRUCTION) and self:isTalentActive(self.T_ELEMENTAL_DESTRUCTION) then
			bt[DamageType.FIRE] = true
			bt[DamageType.LIGHTNING] = true
			bt[DamageType.COLD] = true
			bt[DamageType.ACID] = true
			bt[DamageType.NATURE] = true
			bt[DamageType.BLIGHT] = true
			bt[DamageType.LIGHT] = true
			bt[DamageType.DARKNESS] = true
			bt[DamageType.ARCANE] = true
			bt[DamageType.MIND] = true
			bt[DamageType.TEMPORAL] = true
		else
			if weapon.wielder then
				if weapon.wielder.resists then for res, v in pairs(weapon.wielder.resists) do if v > 0 then bt[res] = true end end end
				if weapon.wielder.on_melee_hit then for res, v in pairs(weapon.wielder.on_melee_hit) do if v > 0 then bt[res] = true end end end
			end
			if weapon.combat then
				if weapon.combat and weapon.combat.melee_project then for res, v in pairs(weapon.combat.melee_project) do if v > 0 then bt[res] = true end end end
				if weapon.combat and weapon.combat.burst_on_crit then for res, v in pairs(weapon.combat.burst_on_crit) do if v > 0 then bt[res] = true end end end
				if weapon.combat and weapon.combat.convert_damage then for res, v in pairs(weapon.combat.convert_damage) do if v > 0 then bt[res] = true end end end
			end
		end

		local n = 0
		for t, _ in pairs(bt) do n = n + 1 end

		if n < 1 then return "(error 2)" end
		local e_string = ""
		if n == 1 then
			e_string = DamageType.dam_def[next(bt)].name
		else
			local list = table.keys(bt)
			for i = 1, #list do
				list[i] = DamageType.dam_def[list[i]].name
			end
			e_string = table.concat(list, ", ")
		end
		return bt, e_string
	end,
	action = function(self, t)
		local properties = t.getProperties(self, t)
		local bt, bt_string = t.getBlockedTypes(self, t)
		self:setEffect(self.EFF_BLOCKING, 1 + (self:knowTalent(self.T_ETERNAL_GUARD) and 1 or 0), {power = t.getBlockValue(self, t), d_types=bt, properties=properties})
		return true
	end,
	info = function(self, t)
		local properties = t.getProperties(self, t)
		local bt, bt_string = t.getBlockedTypes(self, t)
		return ([[Raise your weapon into a parrying position for one turn, reducing the damage of all %s attacks by %d. If you block all of an attack's damage, the attacker will be vulnerable to a deadly counterstrike (a normal attack will instead deal 200%% damage) for one turn.]]):format(bt_string, t.getBlockValue(self, t))
	end,
}

newTalent{
	name = "Master Deflection",
	type = {"technique/martial-mastery", 4},
	require = dex_req_high4,
	no_energy = true,
	cooldown = 25,
	points = 5,
	tactical = { BUFF = 2, DEFEND = 2 },
	on_pre_use = function(self, t, silent) if not self:getInven("MAINHAND") then if not silent then game.logPlayer(self, "You require a mainhand weapon to use this talent.") end return false end return true end,
	action = function(self, t)
		local dur = self:getTalentLevel(t) * 2
		self:setEffect(self.EFF_MASTER_DEFLECTION, dur, {power=10})
		return true
	end,
	info = function(self, t)
		local dur = self:getTalentLevel(t) * 2
		return ([[Through intense focus, the user uses their mainhand weapon to deflect incoming ranged attacks and reflectable spells.  For %d turns the user replaces their ranged defense with accuracy and rolls accuracy vs spellpower to deflect incoming reflectable spells harmlessly.]]):format(dur)
	end,
}

newTalent{
	name = "Dragon Stance",
	type = {"technique/martial-stances", 1},
	points = 1,
	mode = "sustained",
	sustain_stamina = 30,
	cooldown = 30,
	activate = function(self,t)
		if self:knowTalent(self.T_BEAR_STANCE) and self:isTalentActive(self.T_BEAR_STANCE) then
			self:forceUseTalent(self.T_BEAR_STANCE, {ignore_energy=true, ignore_ressources=true})
		end
		if self:knowTalent(self.T_FALCON_STANCE) and self:isTalentActive(self.T_FALCON_STANCE) then
			self:forceUseTalent(self.T_FALCON_STANCE, {ignore_energy=true, ignore_ressources=true})
		end
		local ws = self:getTalentFromId(self.T_WEAPON_STANCE)
		local talent_level = self.getTalentLevel(self,ws)
		return {
			dam_bonus = self:addTemporaryValue("combat_dam", talent_level * 10),
			crit_bonus = self:addTemporaryValue("combat_physcrit", talent_level * 5),
			atk_penalty = self:addTemporaryValue("combat_atk", -talent_level * 5),
			def_penalty = self:addTemporaryValue("combat_def", -talent_level * 5)
		}
	end,
	deactivate = function(self,t,p)
		self:removeTemporaryValue("combat_dam", p.dam_bonus)
		self:removeTemporaryValue("combat_physcrit", p.crit_bonus)
		self:removeTemporaryValue("combat_atk", p.atk_penalty)
		self:removeTemporaryValue("combat_def", p.def_penalty)
		return true
	end,
	info = function(self, t)
		local ws = self:getTalentFromId(self.T_WEAPON_STANCE)
		local talent_level = self.getTalentLevel(self,ws)
		return ([[The user takes up the martial stance of the Dragon.
		It increases the user's physical power by %d and physical critical chance by %d%%.
		This is at the cost of a reduction in accuracy by -%d and defense by -%d.
		This stance will deactivate Bear and Falcon stances.]])
		:format(talent_level * 10,talent_level * 5,talent_level * 5,talent_level * 5)
	end,
}

newTalent{
	name = "Bear Stance",
	type = {"technique/martial-stances", 1},
	points = 1,
	mode = "sustained",
	sustain_stamina = 30,
	cooldown = 30,
	activate = function(self,t)
		if self:knowTalent(self.T_DRAGON_STANCE) and self:isTalentActive(self.T_DRAGON_STANCE) then
			self:forceUseTalent(self.T_DRAGON_STANCE, {ignore_energy=true, ignore_ressources=true})
		end
		if self:knowTalent(self.T_FALCON_STANCE) and self:isTalentActive(self.T_FALCON_STANCE) then
			self:forceUseTalent(self.T_FALCON_STANCE, {ignore_energy=true, ignore_ressources=true})
		end
		local ws = self:getTalentFromId(self.T_WEAPON_STANCE)
		local talent_level = self.getTalentLevel(self,ws)
		return {
			def_bonus = self:addTemporaryValue("combat_def", talent_level*10),
			spell_bonus = self:addTemporaryValue("combat_spellresist", talent_level*6),
			mental_bonus = self:addTemporaryValue("combat_mentalresist", talent_level*6),
			physical_bonus = self:addTemporaryValue("combat_physresist", talent_level*6),
			crit_penalty = self:addTemporaryValue("combat_physcrit", -talent_level*2.5),
			speed_penalty = self:addTemporaryValue("combat_physspeed", -talent_level*0.02)
		}
	end,
	deactivate = function(self,t,p)
		self:removeTemporaryValue("combat_def", p.def_bonus)
		self:removeTemporaryValue("combat_spellresist", p.spell_bonus)
		self:removeTemporaryValue("combat_mentalresist", p.mental_bonus)
		self:removeTemporaryValue("combat_physresist", p.physical_bonus)
		self:removeTemporaryValue("combat_physcrit", p.crit_penalty)
		self:removeTemporaryValue("combat_physspeed", p.speed_penalty)
		return true
	end,
	info = function(self, t)
		local ws = self:getTalentFromId(self.T_WEAPON_STANCE)
		local talent_level = self.getTalentLevel(self,ws)
		return ([[The user takes up the martial stance of the Bear.
		It increases the user's defense by %d and all saves %d.
		This is at the cost of a reduction in physical critical chance by -%d%% and physical attack speed by -%d%%.
		This stance will deactivate Dragon and Falcon stances.]])
		:format(talent_level * 10,talent_level * 6, talent_level * 2.5, talent_level*2)
	end,
}

newTalent{
	name = "Falcon Stance",
	type = {"technique/martial-stances", 1},
	points = 1,
	mode = "sustained",
	sustain_stamina = 30,
	cooldown = 30,
	activate = function(self,t)
		if self:knowTalent(self.T_DRAGON_STANCE) and self:isTalentActive(self.T_DRAGON_STANCE) then
			self:forceUseTalent(self.T_DRAGON_STANCE, {ignore_energy=true, ignore_ressources=true})
		end
		if self:knowTalent(self.T_BEAR_STANCE) and self:isTalentActive(self.T_BEAR_STANCE) then
			self:forceUseTalent(self.T_BEAR_STANCE, {ignore_energy=true, ignore_ressources=true})
		end
		local ws = self:getTalentFromId(self.T_WEAPON_STANCE)
		local talent_level = self.getTalentLevel(self,ws)
		return {
			speed_bonus = self:addTemporaryValue("combat_physspeed", talent_level*0.04),
			atk_bonus = self:addTemporaryValue("combat_atk", talent_level * 10),
			dam_penalty = self:addTemporaryValue("combat_dam", -talent_level*5),
			spell_penalty = self:addTemporaryValue("combat_spellresist", -talent_level*3),
			mental_penalty = self:addTemporaryValue("combat_mentalresist", -talent_level*3),
			physical_penalty = self:addTemporaryValue("combat_physresist", -talent_level*3)
		}
	end,
	deactivate = function(self,t,p)
		self:removeTemporaryValue("combat_physspeed", p.speed_bonus)
		self:removeTemporaryValue("combat_atk", p.atk_bonus)
		self:removeTemporaryValue("combat_dam", p.dam_penalty)
		self:removeTemporaryValue("combat_spellresist", p.spell_penalty)
		self:removeTemporaryValue("combat_mentalresist", p.mental_penalty)
		self:removeTemporaryValue("combat_physresist", p.physical_penalty)
		return true
	end,
	info = function(self, t)
		local ws = self:getTalentFromId(self.T_WEAPON_STANCE)
		local talent_level = self.getTalentLevel(self,ws)
		return ([[The user takes up the martial stance of the Falcon.
		It increases the user's accuracy by %d and physical attack speed by %d%%.
		This is at the cost of a reduction in physical power by -%d and all saves by -%d.
		This stance will deactivate Dragon and Bear stances.]])
		:format(talent_level * 10,talent_level * 4, talent_level * 2.5, talent_level*2)
	end,
}