-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009, 2010, 2011 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

local Stats = require "engine.interface.ActorStats"
local Particles = require "engine.Particles"
local Entity = require "engine.Entity"
local Chat = require "engine.Chat"
local Map = require "engine.Map"
local Level = require "engine.Level"

local _M = loadPrevious(...)

newEffect{
	name = "PURIFY_FIRE", image = "talents/providence.png",
	desc = "Purifying Fire",
	long_desc = function(self, eff) return ("The target is is having its beneficial effects burned away in purifying fire."):format(eff.power) end,
	type = "magical",
	subtype = { fire=true },
	status = "detrimental",
	parameters = {},
	on_merge = function(self,old_eff,new_eff)
		-- Keep the longer duration purifying fire
		if new_eff.dur > old_eff.dur then
			old_eff.dur = new_eff.dur
		end
		return old_eff
	end,
	on_timeout = function(self, eff)
		local effs = {}
		-- Go through all spell effects
		for eff_id, p in pairs(self.tmp) do
			local e = self.tempeffect_def[eff_id]
			if e.status == "beneficial" and e.type ~= "other" then
				effs[#effs+1] = {"effect", eff_id}
			end
		end

		-- Go through all sustained spells
		for tid, act in pairs(self.sustain_talents) do
			if act then
				effs[#effs+1] = {"talent", tid}
			end
		end

		if #effs > 0 then
			local eff = rng.tableRemove(effs)
			if eff[1] == "effect" then
				self:removeEffect(eff[2])
			else
				self:forceUseTalent(eff[2], {ignore_energy=true})
			end
		end
	end,
	activate = function(self, eff)
	end,
	deactivate = function(self, eff)
	end,
}


newEffect{
	name = "WIND_SLOW", image = "effects/hurricane.png",
	desc = "Wind Storm",
	long_desc = function(self, eff) return ("The wind storm reduces the target's global speed by %d%%."):format(eff.power * 100) end,
	type = "magical",
	subtype = { slow=true },
	status = "detrimental",
	parameters = { power = 0.05},
	on_gain = function(self, err) return "#F53CBE##Target# moves slowly against the wind!", "+Slow" end,
	on_lose = function(self, err) return "#Target# overcomes the wind.", "-Slow" end,
	activate = function(self, eff)
		eff.particle = self:addParticles(Particles.new("gloom_slow", 1))
		eff.tmpid = self:addTemporaryValue("global_speed_add", -eff.power)
	end,
	deactivate = function(self, eff)
		self:removeTemporaryValue("global_speed_add", eff.tmpid)
		self:removeParticles(eff.particle)
	end,
}

newEffect{
	name = "TORNADO_STRIKE_PRECISION", image = "talents/tornado_strike.png",
	desc = "Tornado strike precision",
	long_desc = function(self, eff) return ("The user is more precise, imporving accuracy by %d, APR by %d, and crit chance by %d%%."):format(eff.atk, eff.APR, eff.crit) end,
	type = "magical",
	subtype = { lightning=true },
	status = "beneficial",
	parameters = {atk = 1, APR = 1, crit = 1},
	on_gain = function(self, err) return "#Target# is more precise!" end,
	on_lose = function(self, err) return "#Target# is moving normally." end,
	activate = function(self, eff)
		eff.atkid = self:addTemporaryValue("combat_atk", eff.atk)
		eff.APRid = self:addTemporaryValue("combat_apr", eff.APR)
		eff.critid = self:addTemporaryValue("combat_physcrit", eff.crit)
	end,
	deactivate = function(self, eff)
		self:removeTemporaryValue("combat_atk", eff.atkid)
		self:removeTemporaryValue("combat_apr", eff.APRid)
		self:removeTemporaryValue("combat_physcrit", eff.critid)
	end,
}


newEffect{
	name = "MARK_OF_FIRE", image = "effects/inferno.png",
	desc = "Mark of fire",
	long_desc = function(self, eff) return ("The mark of fire is increasing damage dealt by %d%% and slowing it by 50%%."):format(50) end,
	type = "magical",
	subtype = { fire=true },
	status = "detrimental",
	parameters = { power = 1.5},
	activate = function(self, eff)
		eff.tmpid = self:addTemporaryValue("mark_of_fire", eff.power)
		eff.speedid = self:addTemporaryValue("movement_speed", -0.5)
	end,
	deactivate = function(self, eff)
		self:removeTemporaryValue("mark_of_fire", eff.tmpid)
		self:removeTemporaryValue("movement_speed", eff.speedid)
	end,
}


newEffect{
	name = "HEALING_WATERS", image = "talents/healing_water.png",
	desc = "Healing Water",
	long_desc = function(self, eff) return ("Residual healing waters remains and can still heal for %d."):format(eff.power) end,
	type = "magical",
	subtype = { water=true },
	status = "beneficial",
	parameters = { power = 0},
	activate = function(self, eff)
	end,
	deactivate = function(self, eff)
	end,
	on_timeout = function(self, eff)
		local healTotal = eff.power
		local healingNeeded = (self.max_life - self.life) / util.bound((self.healing_factor or 1),0.1,2.5)
		self:heal(math.min(healTotal,healingNeeded), self)
		if healTotal - healingNeeded > 0 then
			eff.power = healTotal - healingNeeded
		else
			eff.power = 0
			eff.dur = 0
		end
	end,
}
